'*** File Details *****************************************************************
'*   File        : Bass.vb
'*   Author      : Rick Ratayczak - rick@karaokedj.net
'*   Created On  : 10/15/2003
'* 
'*   Requires    : BASS.DLL 2.0 - available @ www.un4seen.com
'*                 See the BASS.CHM file for more complete documentation
'*   Type        : VB.NET Class
'*   Description : BASS 2.0 Visual Basic API Header File
'*                 
'*   Copyright   :  1999 - 2003 Ian Luck
'*
'**********************************************************************************

'*** Change Log *******************************************************************
'*
'*   10/15/2003 Initial Creation
'*
'**********************************************************************************
'Option Strict Off
#Region " Imports "

Imports System.Runtime.InteropServices

#End Region



Public Class Bass


    ' BASSCD 2.2 Visual Basic API, copyright (c) 2003-2005 Ian Luck.
    ' Requires BASS 2.2 - available @ www.un4seen.com

    ' See the BASSCD.CHM file for more complete documentation



    ' Additional error codes returned by BASS_ErrorGetCode
    Public Const BASS_ERROR_NOCD As Integer = 12      ' no CD in drive
    Public Const BASS_ERROR_CDTRACK As Integer = 13  ' invalid track number
    ' Public Const BASS_ERROR_NOTAUDIO As Integer = 17 ' not an audio track

    ' Additional config option
    Public Const BASS_CONFIG_CD_FREEOLD As Integer = &H10200

    Public Structure BASS_CD_INFO
        Dim rwflags As Long       'read/write capability flags
        Dim canopen As Long       'BASS_CD_DOOR_OPEN/CLOSE is supported?
        Dim canlock As Long       'BASS_CD_DOOR_LOCK/UNLOCK is supported?
        Dim maxspeed As Long      'max read speed (KB/s)
        Dim cache As Long         'cache size (KB)
        Dim cdtext As Long        'can read CD-TEXT
    End Structure

    ' "rwflag" read capability flags
    Public Const BASS_CD_RWFLAG_READCDR As Integer = 1
    Public Const BASS_CD_RWFLAG_READCDRW As Integer = 2
    Public Const BASS_CD_RWFLAG_READCDRW2 As Integer = 4
    Public Const BASS_CD_RWFLAG_READDVD As Integer = 8
    Public Const BASS_CD_RWFLAG_READDVDR As Integer = 16
    Public Const BASS_CD_RWFLAG_READDVDRAM As Integer = 32
    Public Const BASS_CD_RWFLAG_READANALOG As Integer = &H10000
    Public Const BASS_CD_RWFLAG_READM2F1 As Integer = &H100000
    Public Const BASS_CD_RWFLAG_READM2F2 As Integer = &H200000
    Public Const BASS_CD_RWFLAG_READMULTI As Integer = &H400000
    Public Const BASS_CD_RWFLAG_READCDDA As Integer = &H1000000
    Public Const BASS_CD_RWFLAG_READCDDASIA As Integer = &H2000000
    Public Const BASS_CD_RWFLAG_READSUBCHAN As Integer = &H4000000
    Public Const BASS_CD_RWFLAG_READSUBCHANDI As Integer = &H8000000
    Public Const BASS_CD_RWFLAG_READISRC As Integer = &H20000000
    Public Const BASS_CD_RWFLAG_READUPC As Integer = &H40000000

    Public Const BASS_CD_SUBCHANNEL As Integer = &H200
    Public Const BASS_CD_SUBCHANNEL_NOHW As Integer = &H400

    ' additional CD sync type
    Public Const BASS_SYNC_CD_ERROR As Integer = 1000

    ' BASS_CD_Door actions
    Public Const BASS_CD_DOOR_CLOSE As Integer = 0
    Public Const BASS_CD_DOOR_OPEN As Integer = 1
    Public Const BASS_CD_DOOR_LOCK As Integer = 2
    Public Const BASS_CD_DOOR_UNLOCK As Integer = 3

    ' BASS_CD_GetID flags
    Public Const BASS_CDID_UPC As Integer = 1
    Public Const BASS_CDID_CDDB As Integer = 2
    Public Const BASS_CDID_CDDB2 As Integer = 3
    Public Const BASS_CDID_TEXT As Integer = 4
    Public Const BASS_CDID_CDPLAYER As Integer = 5
    Public Const BASS_CDID_ISRC As Integer = &H100  ' + track #

    ' BASS_CHANNELINFO type
    Public Const BASS_CHANNEL_STREAM_CD As Integer = &H10200


    Declare Function BASS_CD_GetDriveDescription Lib "basscd.dll" (ByVal drive As Integer) As Integer
    Declare Function BASS_CD_GetDriveLetter Lib "basscd.dll" (ByVal drive As Integer) As Integer
    Declare Function BASS_CD_GetInfo Lib "basscd.dll" (ByVal drive As Integer, ByRef info As BASS_CD_INFO) As Integer
    Declare Function BASS_CD_Door Lib "basscd.dll" (ByVal drive As Integer, ByVal action As Integer) As Integer
    Declare Function BASS_CD_DoorIsLocked Lib "basscd.dll" (ByVal drive As Integer) As Integer
    Declare Function BASS_CD_DoorIsOpen Lib "basscd.dll" (ByVal drive As Integer) As Integer
    Declare Function BASS_CD_IsReady Lib "basscd.dll" (ByVal drive As Integer) As Integer
    Declare Function BASS_CD_GetTracks Lib "basscd.dll" (ByVal drive As Integer) As Integer
    Declare Function BASS_CD_GetTrackLength Lib "basscd.dll" (ByVal drive As Integer, ByVal track As Integer) As Integer
    Declare Function BASS_CD_GetID Lib "basscd.dll" (ByVal drive As Integer, ByVal id As Integer) As String
    Declare Function BASS_CD_Release Lib "basscd.dll" (ByVal drive As Integer) As Integer

    Declare Function BASS_CD_StreamCreate Lib "basscd.dll" (ByVal drive As Integer, ByVal track As Integer, ByVal flags As Integer) As Integer
    Declare Function BASS_CD_StreamCreateFile Lib "basscd.dll" (ByVal f As String, ByVal flags As Integer) As Integer
    Declare Function BASS_CD_StreamGetTrack Lib "basscd.dll" (ByVal handle As Integer) As Integer
    Declare Function BASS_CD_StreamSetTrack Lib "basscd.dll" (ByVal handle As Integer, ByVal track As Integer) As Integer

    Declare Function BASS_CD_Analog_Play Lib "basscd.dll" (ByVal drive As Integer, ByVal track As Integer, ByVal pos As Integer) As Integer
    Declare Function BASS_CD_Analog_PlayFile Lib "basscd.dll" (ByVal f As String, ByVal pos As Integer) As Integer
    Declare Function BASS_CD_Analog_Stop Lib "basscd.dll" (ByVal drive As Integer) As Integer
    Declare Function BASS_CD_Analog_IsActive Lib "basscd.dll" (ByVal drive As Integer) As Integer
    Declare Function BASS_CD_Analog_GetPosition Lib "basscd.dll" (ByVal drive As Integer) As Integer




#Region " BASSWMA Constants Declarations "


    ' Additional tags available from BASS_StreamGetTags
    Public Const BASS_TAG_APE As Integer = 6           ' APE tags

    ' Additional tags available from BASS_StreamGetTags
    Public Const BASS_TAG_MP4 As Integer = 7           ' MP4/iTunes metadata

    Const BASS_AAC_DOWNMATRIX As Integer = &H20000      ' downmatrix to 2 channels

    ' BASS_CHANNELINFO type
    Public Const BASS_CTYPE_STREAM_AAC As Integer = &H10B00     ' AAC
    Public Const BASS_CTYPE_STREAM_MP4 As Integer = &H10B01     ' MP4

    ' Additional error codes returned by BASS_WMA_GetErrorCode
    Public Const BASS_ERROR_WMA_LICENSE As Integer = 1000  ' the file is protected
    Public Const BASS_ERROR_WMA_WM9 As Short = 1001    ' WM9 is required
    Public Const BASS_ERROR_WMA_DENIED As Short = 1002    ' access denied (user/pass is invalid)

    ' Additional flags for use with BASS_WMA_EncodeOpenFile/Network
    Public Const BASS_WMA_ENCODE_TAGS As Integer = &H10000   ' set tags in the WMA encoding
    Public Const BASS_WMA_ENCODE_SCRIPT As Integer = &H20000  ' set script (mid-stream tags) in the WMA encoding

    ' Additional flag for use with BASS_WMA_EncodeGetRates
    Public Const BASS_WMA_ENCODE_RATES_VBR As Integer = &H10000    ' get available VBR quality settings

    ' WMENCODEPROC "type" values
    Public Const BASS_WMA_ENCODE_HEAD As Short = 0
    Public Const BASS_WMA_ENCODE_DATA As Short = 1
    Public Const BASS_WMA_ENCODE_DONE As Short = 2

#End Region

#Region " BASSWMA Public Enumerations "

    Public Enum WMACreateFile
        BASS_CREATEFILE_DEFAULT = 0    ' Default Flags
        BASS_SAMPLE_FLOAT = 256    ' 32-bit floating-point
        BASS_SAMPLE_3D = 8    ' 3D functionality enabled
        BASS_SAMPLE_LOOP = 4    ' looped
        BASS_SAMPLE_FX = 128    ' the DX8 effects are enabled
        BASS_SAMPLE_SOFTWARE = 16    ' it's NOT using hardware mixing
        BASS_STREAM_AUTOFREE = 262144    ' automatically free the stream when it stop/ends
        BASS_STREAM_DECODE = &H200000    ' don't play the stream, only decode (BASS_ChannelGetData)
        BASS_SPEAKER_FRONT = &H1000000    ' front speakers
        BASS_SPEAKER_REAR = &H2000000    ' rear/side speakers
        BASS_SPEAKER_CENLFE = &H3000000    ' center & LFE speakers (5.1)
        BASS_SPEAKER_REAR2 = &H4000000    ' rear center speakers (7.1)
        BASS_SPEAKER_LEFT = &H10000000    ' modifier: left
        BASS_SPEAKER_RIGHT = &H20000000    ' modifier: right
        BASS_SPEAKER_FRONTLEFT = BASS_SPEAKER_FRONT Or BASS_SPEAKER_LEFT
        BASS_SPEAKER_FRONTRIGHT = BASS_SPEAKER_FRONT Or BASS_SPEAKER_RIGHT
        BASS_SPEAKER_REARLEFT = BASS_SPEAKER_REAR Or BASS_SPEAKER_LEFT
        BASS_SPEAKER_REARRIGHT = BASS_SPEAKER_REAR Or BASS_SPEAKER_RIGHT
        BASS_SPEAKER_CENTER = BASS_SPEAKER_CENLFE Or BASS_SPEAKER_LEFT
        BASS_SPEAKER_LFE = BASS_SPEAKER_CENLFE Or BASS_SPEAKER_RIGHT
        BASS_SPEAKER_REAR2LEFT = BASS_SPEAKER_REAR2 Or BASS_SPEAKER_LEFT
        BASS_SPEAKER_REAR2RIGHT = BASS_SPEAKER_REAR2 Or BASS_SPEAKER_RIGHT
    End Enum    ' WMACreateFile

#End Region

#Region " BASSWMA API Declarations "

    Public Const BASS_CHANNEL_STREAM_FLAC As Integer = &H10900


    Declare Function BASS_FLAC_StreamCreateFile Lib "bassflac.dll" (ByVal mem As Integer, ByVal f As String, ByVal offset As Integer, ByVal length As Integer, ByVal flags As Integer) As Integer
    Declare Function BASS_FLAC_StreamCreateURL Lib "bassflac.dll" (ByVal url As String, ByVal offset As Integer, ByVal flags As Integer, ByVal proc As Integer, ByVal user As Integer) As Integer
    Declare Function BASS_FLAC_StreamCreateFileUser Lib "bassflac.dll" (ByVal buffered As Integer, ByVal flags As Integer, ByVal proc As Integer, ByVal user As Integer) As Integer

    Declare Function BASS_WMA_StreamCreateFile Lib "basswma.dll" (ByVal mem As Bass.BassBool, ByVal file As String, ByVal offset As Integer, ByVal Length As Integer, ByVal flags As WMACreateFile) As Integer
    'Create a sample stream from a WMA file (or URL).
    'mem    : TRUE = Stream file from memory
    'file   : Filename (mem=FALSE) orelse memory location (mem=TRUE)
    'offset : ignored (set to 0)
    'length : Data length (only used if mem=TRUE)
    'flags  : flags (BASS_SAMPLE_LOOP / BASS_SAMPLE_3D / BASS_SAMPLE_FX / BASS_STREAM_DECODE)
    'RETURN : The created stream's handle (0=error)

    Private Declare Function BASS_WMA_ChannelSetSync Lib "basswma.dll" (ByVal handle As Integer, ByVal atype As Integer, ByVal param As Long, ByVal proc As Integer, ByVal user As Integer) As Integer
    'Setup a sync on a WMA channel. Multiple syncs may be used per channel.
    'handle : Channel handle
    'atype  : Sync type (BASS_SYNC_xxx type & flags)
    'param  : Sync parameters (see the BASS_SYNC_xxx type description)
    'proc   : User defined callback function
    'user   : The 'user' value passed to the callback function
    'RETURN : Sync handle (0=error)

    Declare Function BASS_WMA_GetIWMReader Lib "basswma.dll" (ByVal handle As Integer) As Integer
    'Retrieve the IWMReader interface of a WMA stream. This allows direct
    'access to the WMFSDK functions.
    'handle : channel handle
    'RETURN : Pointer to the IWMReader object interface (0=error)

    Declare Function BASS_WMA_EncodeGetRates Lib "basswma.dll" (ByVal freq As Integer, ByVal flags As Integer) As Integer
    'Retrieve a list of the encoding bitrates available for a
    'specified input sample format.
    'freq   : Sampling rate
    'flags  : BASS_SAMPLE_MONO/BASS_WMA_ENCODE_RATES_VBR flag
    'RETURN : Pointer to an array of bitrates, terminated by a 0 (0=error)

    Declare Function BASS_WMA_EncodeOpen Lib "basswma.dll" (ByVal freq As Integer, ByVal flags As Integer, ByVal bitrate As Integer, ByVal proc As WMENCODEPROC_handler, ByVal user As Integer) As Integer
    'Initialize WMA encoding to a file.
    'freq   : Sampling rate
    'flags  : BASS_SAMPLE_MONO/BASS_WMA_ENCODE_TAGS/BASS_WMA_ENCODE_SCRIPT flags
    'bitrate: Encoding bitrate
    'proc   : AddressOf Callback procedure
    'user   : The 'user' value passed to the callback function */
    'RETURN : The created encoder's handle (0=error)
    'Declare Sub BASS_WMA_EncodeClose Lib "basswma.dll" (ByVal handle As Integer)
    Declare Function BASS_WMA_EncodeOpenFile Lib "basswma.dll" (ByVal freq As Integer, ByVal chans As Integer, ByVal flags As Integer, ByVal bitrate As Integer, ByVal file As String) As Integer
    'Initialize WMA encoding to a file.
    'freq   : Sampling rate
    'flags  : BASS_SAMPLE_MONO/BASS_WMA_ENCODE_TAGS/BASS_WMA_ENCODE_SCRIPT flags
    'bitrate: Encoding bitrate
    'file   : Filename
    'RETURN : The created encoder's handle (0=error)

    Declare Function BASS_WMA_EncodeOpenNetwork Lib "basswma.dll" (ByVal freq As Integer, ByVal flags As Integer, ByVal bitrate As Integer, ByVal port As Integer, ByVal clients As Integer) As Integer
    'Initialize WMA encoding to the network.
    'freq   : Sampling rate
    'flags  : BASS_SAMPLE_MONO/BASS_WMA_ENCODE_TAGS/BASS_WMA_ENCODE_SCRIPT flags
    'bitrate: Encoding bitrate
    'port   : Port number for clients to conenct to (0=let system choose)
    'clients: Maximum number of clients that can connect
    'RETURN : The created encoder's handle (0=error)

    Declare Function BASS_WMA_EncodeOpenPublish Lib "basswma.dll" (ByVal freq As Integer, ByVal flags As Integer, ByVal bitrate As Integer, ByVal url As String, ByVal user As String, ByVal pass As String) As Integer
    'Initialize WMA encoding to a publishing point.
    'freq   : Sampling rate
    'flags  : BASS_SAMPLE_MONO/BASS_WMA_ENCODE_TAGS/BASS_WMA_ENCODE_SCRIPT flags
    'bitrate: Encoding bitrate
    'url    : URL to publish to
    'user   : User Name
    'pass   : Password
    'RETURN : The created encoder's handle (0=error)

    Declare Function BASS_WMA_EncodeGetPort Lib "basswma.dll" (ByVal handle As Integer) As Integer
    'Retrieve the port for clients to connect to a network encoder.
    'handle : Encoder handle
    'RETURN : The port number for clients to connect to (0=error)

    Declare Function BASS_WMA_EncodeSetNotify Lib "basswma.dll" (ByVal handle As Integer, ByVal proc As CLIENTCONNECTPROC_handler, ByVal user As Integer) As Integer
    'Set a client connection notification callback.
    'handle : Encoder handle
    'proc   : User defined notification function
    'user   : The 'user' value passed to the callback function */

    Declare Function BASS_WMA_EncodeGetClients Lib "basswma.dll" (ByVal handle As Integer) As Integer
    'Retrieve the number of clients connected.
    'handle : Encoder handle
    'RETURN : The number of clients (-1=error)

    Declare Function BASS_WMA_EncodeSetTag Lib "basswma.dll" (ByVal handle As Integer, ByVal tag As String, ByVal text As String) As Integer
    'Set a tag in the encoding. 
    'handle : Encoder handle
    'tag    : The tag (vbNullString=no more header tags)
    'text   : The tag's text

    Declare Function BASS_WMA_EncodeWrite Lib "basswma.dll" (ByVal handle As Integer, ByRef buffer As Byte, ByVal length As Integer) As Integer
    'Encode sample data and write it to the file orelse network.
    'handle : Encoder handle
    'buffer : Buffer containing the sample data
    'length : Number of bytes in the buffer

    Declare Sub BASS_WMA_EncodeClose Lib "basswma.dll" (ByVal handle As Integer)
    'Finish encoding and close the file orelse network port.
    'handle : Encoder handle

#End Region

#Region " BASSWMA Callback Procedures "

    Sub CLIENTCONNECTPROC(ByVal handle As Integer, ByVal connect As Integer, ByVal ip As Integer, ByVal user As Integer)

        'CALLBACK FUNCTION !!!

        ' Client connection notification callback function.
        ' handle : The encoder
        ' connect: TRUE=client is connecting, FALSE=disconnecting
        ' ip     : The client's IP (xxx.xxx.xxx.xxx:port)
        ' user   : The 'user' parameter value given when calling BASS_EncodeSetNotify

    End Sub

    Sub WMENCODEPROC(ByVal handle As Integer, ByVal dtype As Integer, ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer)

        'CALLBACK FUNCTION !!!

        ' Encoder callback function.
        ' handle : The encoder handle
        ' dtype  : The type of data, one of BASS_WMA_ENCODE_xxx values
        ' buffer : The encoded data
        ' length : Length of the data
        ' user   : The 'user' parameter value given when calling BASS_WMA_EncodeOpen

    End Sub

#End Region

#Region " BASSWMA Callback Delegate Functions "

    Public Delegate Sub CLIENTCONNECTPROC_handler(ByVal handle As Integer, ByVal connect As Integer, ByVal ip As Integer, ByVal user As Integer)

    Public Delegate Sub WMENCODEPROC_handler(ByVal handle As Integer, ByVal dtype As Integer, ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer)

#End Region


#Region " BASS Constants Declarations "

    ' BASS_CHANNELINFO types
    Public Const BASS_CHANNEL_SAMPLE As Integer = 1
    Public Const BASS_CHANNEL_RECORD As Integer = 2
    Public Const BASS_CHANNEL_STREAM As Integer = &H10000
    Public Const BASS_CHANNEL_STREAM_WAV As Integer = &H10001
    Public Const BASS_CHANNEL_STREAM_OGG As Integer = &H10002
    Public Const BASS_CHANNEL_STREAM_MP1 As Integer = &H10003
    Public Const BASS_CHANNEL_STREAM_MP2 As Integer = &H10004
    Public Const BASS_CHANNEL_STREAM_MP3 As Integer = &H10005
    Public Const BASS_CHANNEL_MUSIC_MOD As Integer = &H20000
    Public Const BASS_CHANNEL_MUSIC_MTM As Integer = &H20001
    Public Const BASS_CHANNEL_MUSIC_S3M As Integer = &H20002
    Public Const BASS_CHANNEL_MUSIC_XM As Integer = &H20003
    Public Const BASS_CHANNEL_MUSIC_IT As Integer = &H20004
    Public Const BASS_CHANNEL_MUSIC_MO3 As Integer = &H20100 ' mo3 flag

    Public Const BASSTRUE As Integer = 1    'Use this instead of VB Booleans
    Public Const BASSFALSE As Integer = 0    'Use this instead of VB Booleans

    '***********************************************
    '* Error codes returned by BASS_GetErrorCode() *
    '***********************************************
    Public Const BASS_OK As Integer = 0    'all is OK
    Public Const BASS_ERROR_MEM As Integer = 1    'memory error
    Public Const BASS_ERROR_FILEOPEN As Integer = 2    'can't open the file
    Public Const BASS_ERROR_DRIVER As Integer = 3    'can't find a free sound driver
    Public Const BASS_ERROR_BUFLOST As Integer = 4    'the sample buffer was lost - please report this!
    Public Const BASS_ERROR_HANDLE As Integer = 5    'invalid handle
    Public Const BASS_ERROR_FORMAT As Integer = 6    'unsupported format
    Public Const BASS_ERROR_POSITION As Integer = 7    'invalid playback position
    Public Const BASS_ERROR_INIT As Integer = 8    'BASS_Init has not been successfully called
    Public Const BASS_ERROR_START As Integer = 9    'BASS_Start has not been successfully called
    Public Const BASS_ERROR_ALREADY As Integer = 14    'already initialized
    Public Const BASS_ERROR_NOPAUSE As Integer = 16    'not paused
    Public Const BASS_ERROR_NOTAUDIO As Integer = 17    'not an audio track
    Public Const BASS_ERROR_NOCHAN As Integer = 18    'can't get a free channel
    Public Const BASS_ERROR_ILLTYPE As Integer = 19    'an illegal type was specified
    Public Const BASS_ERROR_ILLPARAM As Integer = 20    'an illegal parameter was specified
    Public Const BASS_ERROR_NO3D As Integer = 21    'no 3D support
    Public Const BASS_ERROR_NOEAX As Integer = 22    'no EAX support
    Public Const BASS_ERROR_DEVICE As Integer = 23    'illegal device number
    Public Const BASS_ERROR_NOPLAY As Integer = 24    'not playing
    Public Const BASS_ERROR_FREQ As Integer = 25    'illegal sample rate
    Public Const BASS_ERROR_NOTFILE As Integer = 27    'the stream is not a file stream (WAV/MP3)
    Public Const BASS_ERROR_NOHW As Integer = 29    'no hardware voices available
    Public Const BASS_ERROR_EMPTY As Integer = 31    'the MOD music has no sequence data
    Public Const BASS_ERROR_NONET As Integer = 32    'no internet connection could be opened
    Public Const BASS_ERROR_CREATE As Integer = 33    'couldn't create the file
    Public Const BASS_ERROR_NOFX As Integer = 34    'effects are not enabled
    Public Const BASS_ERROR_PLAYING As Integer = 35    'the channel is playing
    Public Const BASS_ERROR_NOTAVAIL As Integer = 37    'requested data is not available
    Public Const BASS_ERROR_DECODE As Integer = 38    'the channel is a "decoding channel"
    Public Const BASS_ERROR_DX As Integer = 39    'a sufficient DirectX version is not installed
    Public Const BASS_ERROR_TIMEOUT As Integer = 40    'connection timedout
    Public Const BASS_ERROR_FILEFORM As Integer = 41    'unsupported file format
    Public Const BASS_ERROR_SPEAKER As Integer = 42    'unavailable speaker
    Public Const BASS_ERROR_UNKNOWN As Integer = -1    'some other mystery error

    '**********************
    '* Device setup flags *
    '**********************
    Public Const BASS_DEVICE_8BITS As Integer = 1    'use 8 bit resolution, else 16 bit
    Public Const BASS_DEVICE_MONO As Integer = 2    'use mono, else stereo
    Public Const BASS_DEVICE_3D As Integer = 4    'enable 3D functionality
    ' If the BASS_DEVICE_3D flag is not specified when initilizing BASS,
    ' then the 3D flags (BASS_SAMPLE_3D and BASS_MUSIC_3D) are ignored when
    ' loading/creating a sample/stream/music.
    Public Const BASS_DEVICE_LATENCY As Integer = 256    'calculate device latency (BASS_INFO struct)
    Public Const BASS_DEVICE_SPEAKERS As Integer = 2048    'force enabling of speaker assignment

    '***********************************
    '* BASS_INFO flags (from DSOUND.H) *
    '***********************************
    Public Const DSCAPS_CONTINUOUSRATE As Integer = 16
    ' supports all sample rates between min/maxrate
    Public Const DSCAPS_EMULDRIVER As Integer = 32
    ' device does NOT have hardware DirectSound support
    Public Const DSCAPS_CERTIFIED As Integer = 64
    ' device driver has been certified by Microsoft
    ' The following flags tell what type of samples are supported by HARDWARE
    ' mixing, all these formats are supported by SOFTWARE mixing
    Public Const DSCAPS_SECONDARYMONO As Integer = 256    ' mono
    Public Const DSCAPS_SECONDARYSTEREO As Integer = 512    ' stereo
    Public Const DSCAPS_SECONDARY8BIT As Integer = 1024    ' 8 bit
    Public Const DSCAPS_SECONDARY16BIT As Integer = 2048    ' 16 bit

    '*****************************************
    '* BASS_RECORDINFO flags (from DSOUND.H) *
    '*****************************************
    Public Const DSCCAPS_EMULDRIVER As Integer = DSCAPS_EMULDRIVER
    ' device does NOT have hardware DirectSound recording support
    Public Const DSCCAPS_CERTIFIED As Integer = DSCAPS_CERTIFIED
    ' device driver has been certified by Microsoft

    '******************************************************************
    '* defines for formats field of BASS_RECORDINFO (from MMSYSTEM.H) *
    '******************************************************************
    Public Const WAVE_FORMAT_1M08 As Integer = &H1S    ' 11.025 kHz, Mono,   8-bit
    Public Const WAVE_FORMAT_1S08 As Integer = &H2S    ' 11.025 kHz, Stereo, 8-bit
    Public Const WAVE_FORMAT_1M16 As Integer = &H4S    ' 11.025 kHz, Mono,   16-bit
    Public Const WAVE_FORMAT_1S16 As Integer = &H8S    ' 11.025 kHz, Stereo, 16-bit
    Public Const WAVE_FORMAT_2M08 As Integer = &H10S    ' 22.05  kHz, Mono,   8-bit
    Public Const WAVE_FORMAT_2S08 As Integer = &H20S    ' 22.05  kHz, Stereo, 8-bit
    Public Const WAVE_FORMAT_2M16 As Integer = &H40S    ' 22.05  kHz, Mono,   16-bit
    Public Const WAVE_FORMAT_2S16 As Integer = &H80S    ' 22.05  kHz, Stereo, 16-bit
    Public Const WAVE_FORMAT_4M08 As Integer = &H100S    ' 44.1   kHz, Mono,   8-bit
    Public Const WAVE_FORMAT_4S08 As Integer = &H200S    ' 44.1   kHz, Stereo, 8-bit
    Public Const WAVE_FORMAT_4M16 As Integer = &H400S    ' 44.1   kHz, Mono,   16-bit
    Public Const WAVE_FORMAT_4S16 As Integer = &H800S    ' 44.1   kHz, Stereo, 16-bit

    '***************
    '* Music flags *
    '***************
    Public Const BASS_MUSIC_FLOAT As Integer = BASS_SAMPLE_FLOAT    ' 32-bit floating-point
    Public Const BASS_MUSIC_MONO As Integer = BASS_SAMPLE_MONO     ' force mono mixing (less CPU usage)
    Public Const BASS_MUSIC_LOOP As Integer = BASS_SAMPLE_LOOP     ' loop music
    Public Const BASS_MUSIC_3D As Integer = BASS_SAMPLE_3D     ' enable 3D functionality
    Public Const BASS_MUSIC_FX As Integer = BASS_SAMPLE_FX     ' enable old implementation of DX8 effects
    Public Const BASS_MUSIC_AUTOFREE As Integer = BASS_STREAM_AUTOFREE     ' automatically free the music when it stop/ends
    Public Const BASS_MUSIC_DECODE As Integer = BASS_STREAM_DECODE     ' don't play the music, only decode (BASS_ChannelGetData)
    Public Const BASS_MUSIC_RAMP As Integer = &H200      ' normal ramping
    Public Const BASS_MUSIC_RAMPS As Integer = &H400     ' sensitive ramping
    Public Const BASS_MUSIC_SURROUND As Integer = &H800     ' surround sound
    Public Const BASS_MUSIC_SURROUND2 As Integer = &H1000     ' surround sound (mode 2)
    Public Const BASS_MUSIC_FT2MOD As Integer = &H2000     ' play .MOD as FastTracker 2 does
    Public Const BASS_MUSIC_PT1MOD As Integer = &H4000     ' play .MOD as ProTracker 1 does
    Public Const BASS_MUSIC_CALCLEN As Integer = &H8000     ' calculate playback length
    Public Const BASS_MUSIC_NONINTER As Integer = &H10000     ' non-interpolated mixing
    Public Const BASS_MUSIC_POSRESET As Integer = &H20000     ' stop all notes when moving position
    Public Const BASS_MUSIC_STOPBACK As Integer = &H80000     ' stop the music on a backwards jump effect
    Public Const BASS_MUSIC_NOSAMPLE As Integer = &H100000    ' don't load the samples

    '*********************
    '* Sample info flags *
    '*********************
    Public Const BASS_SAMPLE_8BITS As Integer = 1    ' 8 bit, else 16 bit
    Public Const BASS_SAMPLE_FLOAT As Short = 256    ' 32-bit floating-point
    Public Const BASS_SAMPLE_MONO As Integer = 2    ' mono, else stereo
    Public Const BASS_SAMPLE_LOOP As Integer = 4    ' looped
    Public Const BASS_SAMPLE_3D As Integer = 8    ' 3D functionality enabled
    Public Const BASS_SAMPLE_SOFTWARE As Integer = 16    ' it's NOT using hardware mixing
    Public Const BASS_SAMPLE_MUTEMAX As Integer = 32    ' muted at max distance (3D only)
    Public Const BASS_SAMPLE_VAM As Integer = 64    ' uses the DX7 voice allocation & management
    Public Const BASS_SAMPLE_FX As Integer = 128    ' the DX8 effects are enabled
    Public Const BASS_SAMPLE_OVER_VOL As Integer = 65536    ' override lowest volume
    Public Const BASS_SAMPLE_OVER_POS As Integer = 131072    ' override longest playing
    Public Const BASS_SAMPLE_OVER_DIST As Integer = 196608    ' override furthest from listener (3D only)

    Public Const BASS_MP3_SETPOS As Integer = 131072    ' enable pin-point seeking on the MP3/MP2/MP1/OGG

    Public Const BASS_STREAM_AUTOFREE As Integer = 262144    ' automatically free the stream when it stop/ends
    Public Const BASS_STREAM_RESTRATE As Integer = 524288    ' restrict the download rate of internet file streams
    Public Const BASS_STREAM_BLOCK As Integer = 1048576    ' download & play internet file stream (MPx) in small blocks
    Public Const BASS_STREAM_DECODE As Integer = &H200000    ' don't play the stream, only decode (BASS_ChannelGetData)
    Public Const BASS_STREAM_META As Integer = &H400000    ' request metadata from a Shoutcast stream
    Public Const BASS_STREAM_FILEPROC As Integer = &H800000    ' use a STREAMFILEPROC callback

    ' Speaker assignment flags
    Public Const BASS_SPEAKER_FRONT As Integer = &H1000000    ' front speakers
    Public Const BASS_SPEAKER_REAR As Integer = &H2000000    ' rear/side speakers
    Public Const BASS_SPEAKER_CENLFE As Integer = &H3000000    ' center & LFE speakers (5.1)
    Public Const BASS_SPEAKER_REAR2 As Integer = &H4000000    ' rear center speakers (7.1)
    Public Const BASS_SPEAKER_LEFT As Integer = &H10000000    ' modifier: left
    Public Const BASS_SPEAKER_RIGHT As Integer = &H20000000    ' modifier: right
    Public Const BASS_SPEAKER_FRONTLEFT As Integer = BASS_SPEAKER_FRONT Or BASS_SPEAKER_LEFT
    Public Const BASS_SPEAKER_FRONTRIGHT As Integer = BASS_SPEAKER_FRONT Or BASS_SPEAKER_RIGHT
    Public Const BASS_SPEAKER_REARLEFT As Integer = BASS_SPEAKER_REAR Or BASS_SPEAKER_LEFT
    Public Const BASS_SPEAKER_REARRIGHT As Integer = BASS_SPEAKER_REAR Or BASS_SPEAKER_RIGHT
    Public Const BASS_SPEAKER_CENTER As Integer = BASS_SPEAKER_CENLFE Or BASS_SPEAKER_LEFT
    Public Const BASS_SPEAKER_LFE As Integer = BASS_SPEAKER_CENLFE Or BASS_SPEAKER_RIGHT
    Public Const BASS_SPEAKER_REAR2LEFT As Integer = BASS_SPEAKER_REAR2 Or BASS_SPEAKER_LEFT
    Public Const BASS_SPEAKER_REAR2RIGHT As Integer = BASS_SPEAKER_REAR2 Or BASS_SPEAKER_RIGHT


    '**********************************************
    '* BASS_StreamGetTags flags : what's returned *
    '**********************************************
    Public Const BASS_TAG_ID3 As Integer = 0    'ID3v1 tags : 128 byte block
    Public Const BASS_TAG_ID3V2 As Integer = 1    'ID3v2 tags : variable length block
    Public Const BASS_TAG_OGG As Integer = 2    'OGG comments : array of null-terminated strings
    Public Const BASS_TAG_HTTP As Integer = 3    'HTTP headers : array of null-terminated strings
    Public Const BASS_TAG_ICY As Integer = 4    'ICY headers : array of null-terminated strings
    Public Const BASS_TAG_META As Integer = 5    'ICY metadata : null-terminated string

    '********************
    '* 3D channel modes *
    '********************
    Public Const BASS_3DMODE_NORMAL As Integer = 0
    ' normal 3D processing
    Public Const BASS_3DMODE_RELATIVE As Integer = 1
    ' The channel's 3D position (position/velocity/orientation) are relative to
    ' the listener. When the listener's position/velocity/orientation is changed
    ' with BASS_Set3DPosition, the channel's position relative to the listener does
    ' not change.
    Public Const BASS_3DMODE_OFF As Integer = 2
    ' Turn off 3D processing on the channel, the sound will be played
    ' in the center.


    '****************************************************
    '* EAX environments, use with BASS_SetEAXParameters *
    '****************************************************
    Public Const EAX_ENVIRONMENT_GENERIC As Integer = 0
    Public Const EAX_ENVIRONMENT_PADDEDCELL As Integer = 1
    Public Const EAX_ENVIRONMENT_ROOM As Integer = 2
    Public Const EAX_ENVIRONMENT_BATHROOM As Integer = 3
    Public Const EAX_ENVIRONMENT_LIVINGROOM As Integer = 4
    Public Const EAX_ENVIRONMENT_STONEROOM As Integer = 5
    Public Const EAX_ENVIRONMENT_AUDITORIUM As Integer = 6
    Public Const EAX_ENVIRONMENT_CONCERTHALL As Integer = 7
    Public Const EAX_ENVIRONMENT_CAVE As Integer = 8
    Public Const EAX_ENVIRONMENT_ARENA As Integer = 9
    Public Const EAX_ENVIRONMENT_HANGAR As Integer = 10
    Public Const EAX_ENVIRONMENT_CARPETEDHALLWAY As Integer = 11
    Public Const EAX_ENVIRONMENT_HALLWAY As Integer = 12
    Public Const EAX_ENVIRONMENT_STONECORRIDOR As Integer = 13
    Public Const EAX_ENVIRONMENT_ALLEY As Integer = 14
    Public Const EAX_ENVIRONMENT_FOREST As Integer = 15
    Public Const EAX_ENVIRONMENT_CITY As Integer = 16
    Public Const EAX_ENVIRONMENT_MOUNTAINS As Integer = 17
    Public Const EAX_ENVIRONMENT_QUARRY As Integer = 18
    Public Const EAX_ENVIRONMENT_PLAIN As Integer = 19
    Public Const EAX_ENVIRONMENT_PARKINGLOT As Integer = 20
    Public Const EAX_ENVIRONMENT_SEWERPIPE As Integer = 21
    Public Const EAX_ENVIRONMENT_UNDERWATER As Integer = 22
    Public Const EAX_ENVIRONMENT_DRUGGED As Integer = 23
    Public Const EAX_ENVIRONMENT_DIZZY As Integer = 24
    Public Const EAX_ENVIRONMENT_PSYCHOTIC As Integer = 25
    ' total number of environments
    Public Const EAX_ENVIRONMENT_COUNT As Integer = 26

    '*************************************************************
    '* EAX presets, usage: BASS_SetEAXParametersVB(EAX_PRESET_xxx) *
    '*************************************************************
    Public Const EAX_PRESET_GENERIC As Integer = 1
    Public Const EAX_PRESET_PADDEDCELL As Integer = 2
    Public Const EAX_PRESET_ROOM As Integer = 3
    Public Const EAX_PRESET_BATHROOM As Integer = 4
    Public Const EAX_PRESET_LIVINGROOM As Integer = 5
    Public Const EAX_PRESET_STONEROOM As Integer = 6
    Public Const EAX_PRESET_AUDITORIUM As Integer = 7
    Public Const EAX_PRESET_CONCERTHALL As Integer = 8
    Public Const EAX_PRESET_CAVE As Integer = 9
    Public Const EAX_PRESET_ARENA As Integer = 10
    Public Const EAX_PRESET_HANGAR As Integer = 11
    Public Const EAX_PRESET_CARPETEDHALLWAY As Integer = 12
    Public Const EAX_PRESET_HALLWAY As Integer = 13
    Public Const EAX_PRESET_STONECORRIDOR As Integer = 14
    Public Const EAX_PRESET_ALLEY As Integer = 15
    Public Const EAX_PRESET_FOREST As Integer = 16
    Public Const EAX_PRESET_CITY As Integer = 17
    Public Const EAX_PRESET_MOUNTAINS As Integer = 18
    Public Const EAX_PRESET_QUARRY As Integer = 19
    Public Const EAX_PRESET_PLAIN As Integer = 20
    Public Const EAX_PRESET_PARKINGLOT As Integer = 21
    Public Const EAX_PRESET_SEWERPIPE As Integer = 22
    Public Const EAX_PRESET_UNDERWATER As Integer = 23
    Public Const EAX_PRESET_DRUGGED As Integer = 24
    Public Const EAX_PRESET_DIZZY As Integer = 25
    Public Const EAX_PRESET_PSYCHOTIC As Integer = 26


    '**********************************************************************
    '* Sync types (with BASS_ChannelSetSync() "param" and SYNCPROC "data" *
    '* definitions) & flags.                                              *
    '**********************************************************************
    ' Sync when a music or stream reaches a position.
    ' if HMUSIC...
    ' param: LOWORD=order (0=first, -1=all) HIWORD=row (0=first, -1=all)
    ' data : LOWORD=order HIWORD=row
    ' if HSTREAM...
    ' param: position in bytes
    ' data : not used
    Public Const BASS_SYNC_POS As Integer = 0
    Public Const BASS_SYNC_MUSICPOS As Integer = 0
    ' Sync when an instrument (sample for the non-instrument based formats)
    ' is played in a music (not including retrigs).
    ' param: LOWORD=instrument (1=first) HIWORD=note (0=c0...119=b9, -1=all)
    ' data : LOWORD=note HIWORD=volume (0-64)
    Public Const BASS_SYNC_MUSICINST As Integer = 1
    ' Sync when a music or file stream reaches the end.
    ' param: not used
    ' data : not used
    Public Const BASS_SYNC_END As Integer = 2
    ' Sync when the "sync" effect (XM/MTM/MOD: E8x/Wxx, IT/S3M: S2x) is used.
    ' param: 0:data=pos, 1:data="x" value
    ' data : param=0: LOWORD=order HIWORD=row, param=1: "x" value
    Public Const BASS_SYNC_MUSICFX As Integer = 3
    ' FLAG: post a Windows message (instead of callback)
    ' When using a window message "callback", the message to post is given in the "proc"
    ' parameter of BASS_ChannelSetSync, and is posted to the window specified in the BASS_Init
    ' call. The message parameters are: WPARAM = data, LPARAM = user.
    Public Const BASS_SYNC_META As Integer = 4
    ' Sync when metadata is received in a Shoutcast stream.
    ' param: not used
    ' data : pointer to the metadata
    Public Const BASS_SYNC_SLIDE As Integer = 5
    ' Sync when an attribute slide is completed.
    ' param: not used
    ' data : the type of slide completed (one of the BASS_SLIDE_xxx values)
    Public Const BASS_SYNC_MESSAGE As Integer = &H20000000
    'FLAG: sync at mixtime, else at playtime
    Public Const BASS_SYNC_MIXTIME As Integer = &H40000000
    ' FLAG: sync only once, else continuously
    Public Const BASS_SYNC_ONETIME As Integer = &H80000000

    Public Const RECORDCHAN As Integer = 1    ' Recording channel, for use with BASS_Channel functions

    ' BASS_ChannelIsActive return values
    Public Const BASS_ACTIVE_STOPPED As Integer = 0
    Public Const BASS_ACTIVE_PLAYING As Integer = 1
    Public Const BASS_ACTIVE_STALLED As Integer = 2
    Public Const BASS_ACTIVE_PAUSED As Integer = 3

    ' BASS_ChannelIsSliding return flags
    Public Const BASS_SLIDE_FREQ As Integer = 1
    Public Const BASS_SLIDE_VOL As Integer = 2
    Public Const BASS_SLIDE_PAN As Integer = 4

    ' BASS_ChannelGetData flags
    Public Const BASS_DATA_AVAILABLE As Integer = 0    ' query how much data is buffered
    Public Const BASS_DATA_FFT512 As Integer = &H80000000    ' 512 sample FFT
    Public Const BASS_DATA_FFT1024 As Integer = &H80000001    ' 1024 FFT
    Public Const BASS_DATA_FFT2048 As Integer = &H80000002    ' 2048 FFT
    Public Const BASS_DATA_FFT_INDIVIDUAL As Integer = &H10    ' FFT flag: FFT for each channel, else all combined
    Public Const BASS_DATA_FFT_NOWINDOW As Integer = &H20   ' FFT flag: no Hanning window

    ' BASS_RecordSetInput flags
    Public Const BASS_INPUT_OFF As Integer = &H10000
    Public Const BASS_INPUT_ON As Integer = &H20000
    Public Const BASS_INPUT_LEVEL As Integer = &H40000

    Public Const BASS_INPUT_TYPE_MASK As Integer = &HFF000000
    Public Const BASS_INPUT_TYPE_UNDEF As Integer = &H0
    Public Const BASS_INPUT_TYPE_DIGITAL As Integer = &H1000000
    Public Const BASS_INPUT_TYPE_LINE As Integer = &H2000000
    Public Const BASS_INPUT_TYPE_MIC As Integer = &H3000000
    Public Const BASS_INPUT_TYPE_SYNTH As Integer = &H4000000
    Public Const BASS_INPUT_TYPE_CD As Integer = &H5000000
    Public Const BASS_INPUT_TYPE_PHONE As Integer = &H6000000
    Public Const BASS_INPUT_TYPE_SPEAKER As Integer = &H7000000
    Public Const BASS_INPUT_TYPE_WAVE As Integer = &H8000000
    Public Const BASS_INPUT_TYPE_AUX As Integer = &H9000000
    Public Const BASS_INPUT_TYPE_ANALOG As Integer = &HA000000

    ' BASS_SetNetConfig flags
    Public Const BASS_NET_TIMEOUT As Integer = 0
    Public Const BASS_NET_BUFFER As Integer = 1

    ' BASS_Set/GetConfig options
    Public Const BASS_CONFIG_BUFFER As Short = 0
    Public Const BASS_CONFIG_UPDATEPERIOD As Short = 1
    Public Const BASS_CONFIG_MAXVOL As Short = 3
    Public Const BASS_CONFIG_GVOL_SAMPLE As Short = 4
    Public Const BASS_CONFIG_GVOL_STREAM As Short = 5
    Public Const BASS_CONFIG_GVOL_MUSIC As Short = 6
    Public Const BASS_CONFIG_CURVE_VOL As Short = 7
    Public Const BASS_CONFIG_CURVE_PAN As Short = 8
    Public Const BASS_CONFIG_FLOATDSP As Short = 9
    Public Const BASS_CONFIG_3DALGORITHM As Short = 10
    Public Const BASS_CONFIG_NET_TIMEOUT As Short = 11
    Public Const BASS_CONFIG_NET_BUFFER As Short = 12

    ' BASS_StreamGetFilePosition modes
    Public Const BASS_FILEPOS_DECODE As Integer = 0
    Public Const BASS_FILEPOS_DOWNLOAD As Integer = 1
    Public Const BASS_FILEPOS_END As Integer = 2

    ' STREAMFILEPROC actions
    Public Const BASS_FILE_CLOSE As Integer = 0
    Public Const BASS_FILE_READ As Integer = 1
    Public Const BASS_FILE_QUERY As Integer = 2
    Public Const BASS_FILE_LEN As Integer = 3

    Public Const BASS_STREAMPROC_END As Integer = &H80000000     ' end of user stream flag

    '**************************************************************
    '* DirectSound interfaces (for use with BASS_GetDSoundObject) *
    '**************************************************************
    Public Const BASS_OBJECT_DS As Integer = 1    ' DirectSound
    Public Const BASS_OBJECT_DS3DL As Integer = 2    'IDirectSound3DListener

    '******************************
    '* DX7 voice allocation flags *
    '******************************
    ' Play the sample in hardware. If no hardware voices are available then
    ' the "play" call will fail
    Public Const BASS_VAM_HARDWARE As Integer = 1
    ' Play the sample in software (ie. non-accelerated). No other VAM flags
    'may be used together with this flag.
    Public Const BASS_VAM_SOFTWARE As Integer = 2

    '******************************
    '* DX7 voice management flags *
    '******************************
    ' These flags enable hardware resource stealing... if the hardware has no
    ' available voices, a currently playing buffer will be stopped to make room for
    ' the new buffer. NOTE: only samples loaded/created with the BASS_SAMPLE_VAM
    ' flag are considered for termination by the DX7 voice management.

    ' If there are no free hardware voices, the buffer to be terminated will be
    ' the one with the least time left to play.
    Public Const BASS_VAM_TERM_TIME As Integer = 4
    ' If there are no free hardware voices, the buffer to be terminated will be
    ' one that was loaded/created with the BASS_SAMPLE_MUTEMAX flag and is beyond
    ' it 's max distance. If there are no buffers that match this criteria, then the
    ' "play" call will fail.
    Public Const BASS_VAM_TERM_DIST As Integer = 8
    ' If there are no free hardware voices, the buffer to be terminated will be
    ' the one with the lowest priority.
    Public Const BASS_VAM_TERM_PRIO As Integer = 16

    '**********************************************************************
    '* software 3D mixing algorithm modes (used with BASS_Set3DAlgorithm) *
    '**********************************************************************
    ' default algorithm (currently translates to BASS_3DALG_OFF)
    Public Const BASS_3DALG_DEFAULT As Integer = 0
    ' Uses normal left and right panning. The vertical axis is ignored except for
    'scaling of volume due to distance. Doppler shift and volume scaling are still
    'applied, but the 3D filtering is not performed. This is the most CPU efficient
    'software implementation, but provides no virtual 3D audio effect. Head Related
    'Transfer Function processing will not be done. Since only normal stereo panning
    'is used, a channel using this algorithm may be accelerated by a 2D hardware
    'voice if no free 3D hardware voices are available.
    Public Const BASS_3DALG_OFF As Integer = 1
    ' This algorithm gives the highest quality 3D audio effect, but uses more CPU.
    ' Requires Windows 98 2nd Edition or Windows 2000 that uses WDM drivers, if this
    ' mode is not available then BASS_3DALG_OFF will be used instead.
    Public Const BASS_3DALG_FULL As Integer = 2
    ' This algorithm gives a good 3D audio effect, and uses less CPU than the FULL
    ' mode. Requires Windows 98 2nd Edition or Windows 2000 that uses WDM drivers, if
    ' this mode is not available then BASS_3DALG_OFF will be used instead.
    Public Const BASS_3DALG_LIGHT As Integer = 3

    ' DX8 effect types, use with BASS_ChannelSetFX
    Public Const BASS_FX_CHORUS As Integer = 0    ' GUID_DSFX_STANDARD_CHORUS
    Public Const BASS_FX_COMPRESSOR As Integer = 1    ' GUID_DSFX_STANDARD_COMPRESSOR
    Public Const BASS_FX_DISTORTION As Integer = 2    ' GUID_DSFX_STANDARD_DISTORTION
    Public Const BASS_FX_ECHO As Integer = 3    ' GUID_DSFX_STANDARD_ECHO
    Public Const BASS_FX_FLANGER As Integer = 4    ' GUID_DSFX_STANDARD_FLANGER
    Public Const BASS_FX_GARGLE As Integer = 5    ' GUID_DSFX_STANDARD_GARGLE
    Public Const BASS_FX_I3DL2REVERB As Integer = 6    ' GUID_DSFX_STANDARD_I3DL2REVERB
    Public Const BASS_FX_PARAMEQ As Integer = 7    ' GUID_DSFX_STANDARD_PARAMEQ
    Public Const BASS_FX_REVERB As Integer = 8    ' GUID_DSFX_WAVES_REVERB

    Public Const BASS_FX_PHASE_NEG_180 As Integer = 0
    Public Const BASS_FX_PHASE_NEG_90 As Integer = 1
    Public Const BASS_FX_PHASE_ZERO As Integer = 2
    Public Const BASS_FX_PHASE_90 As Integer = 3
    Public Const BASS_FX_PHASE_180 As Integer = 4

#End Region

#Region " BASS Public Enumerations "

    Public Enum BassConfigOptions As Short
        ' BASS_Set/GetConfig options
        BASS_CONFIG_BUFFER = 0
        BASS_CONFIG_UPDATEPERIOD = 1
        BASS_CONFIG_MAXVOL = 3
        BASS_CONFIG_GVOL_SAMPLE = 4
        BASS_CONFIG_GVOL_STREAM = 5
        BASS_CONFIG_GVOL_MUSIC = 6
        BASS_CONFIG_CURVE_VOL = 7
        BASS_CONFIG_CURVE_PAN = 8
        BASS_CONFIG_FLOATDSP = 9
        BASS_CONFIG_3DALGORITHM = 10
        BASS_CONFIG_NET_TIMEOUT = 11
        BASS_CONFIG_NET_BUFFER = 12
    End Enum    ' BassConfigOptions

    Public Enum BassBool
        BassFalse = 0
        BassTrue = 1
    End Enum    ' BassBool

    Public Enum Bass_ErrorCodes
        '***********************************************
        '* Error codes returned by BASS_GetErrorCode() *
        '***********************************************
        Bass_Error_Ok = 0    'all Is Ok
        Bass_Error_Mem = 1    'memory Error
        Bass_Error_Fileopen = 2    'can't Open The File
        Bass_Error_Driver = 3    'can't Find A Free Sound Driver
        Bass_Error_Buflost = 4    'the Sample Buffer Was Lost - Please Report This!
        Bass_Error_Handle = 5    'invalid Handle
        Bass_Error_Format = 6    'unsupported Format
        Bass_Error_Position = 7    'invalid Playback Position
        Bass_Error_Init = 8    'bass_Init Has Not Been Successfully Called
        Bass_Error_Start = 9    'bass_Start Has Not Been Successfully Called
        Bass_Error_Already = 14    'already Initialized
        Bass_Error_Nopause = 16    'not Paused
        Bass_Error_Notaudio = 17    'not An Audio Track
        Bass_Error_Nochan = 18    'can't Get A Free Channel
        Bass_Error_Illtype = 19    'an Illegal Type Was Specified
        Bass_Error_Illparam = 20    'an Illegal Parameter Was Specified
        Bass_Error_No3D = 21    'no 3D Support
        Bass_Error_Noeax = 22    'no Eax Support
        Bass_Error_Device = 23    'illegal Device Number
        Bass_Error_Noplay = 24    'not Playing
        Bass_Error_Freq = 25    'illegal Sample Rate
        Bass_Error_Notfile = 27    'the Stream Is Not A File Stream (Wav/Mp3)
        Bass_Error_Nohw = 29    'no Hardware Voices Available
        Bass_Error_Empty = 31    'the Mod Music Has No Sequence Data
        Bass_Error_Nonet = 32    'no Internet Connection Could Be Opened
        Bass_Error_Create = 33    'couldn't Create The File
        Bass_Error_Nofx = 34    'effects Are Not Enabled
        Bass_Error_Playing = 35    'the Channel Is Playing
        Bass_Error_Notavail = 37    'requested Data Is Not Available
        Bass_Error_Decode = 38    'the Channel Is A "Decoding Channel"
        Bass_Error_Dx = 39    'a Sufficient Directx Version Is Not Installed
        Bass_Error_Timeout = 40    'connection Timedout
        Bass_Error_Fileform = 41    'unsupported File Format
        Bass_Error_Speaker = 42    'unavailable Speaker
        Bass_Error_Unknown = -1    'some Other Mystery Error
    End Enum    ' Bass_ErrorCodes

    Public Enum Bass_Tag
        'ID3v1 tags. A pointer to a 128 byte block is returned. See 
        'http://www.id3.org/id3v1.html 
        'for details of the block's structure.  
        Bass_Tag_Id3 = 0
        'ID3v2 tags. A pointer to a variable length block is returned. See 
        'http://www.id3.org/
        'for details of the block's structure.  
        Bass_Tag_Id3V2 = 1
        'OGG comments, only available when streaming an OGG file. A 
        'pointer to a series of null-terminated strings is returned, the 
        'final string ending with a double null.  
        'bass_Tag_Ogg = 2
        'HTTP headers, only available when streaming from a HTTP server. A 
        'pointer to a series of null-terminated strings is returned, the 
        'final string ending with a double null.  
        Bass_Tag_Http = 3
        'ICY (Shoutcast) tags. A pointer to a series of null-terminated 
        'strings is returned, the final string ending with a double null.  
        Bass_Tag_Icy = 4
        'Shoutcast metadata. A single null-terminated string containing 
        'the current stream title and url (usually omitted). The format of 
        'the string is: StreamTitle='xxx';StreamUrl='xxx';  
        Bass_Tag_Meta = 5
    End Enum    ' Bass_Tag

    Public Enum Bass_Device
        '**********************
        '* Device setup flags *
        '**********************
        Device_Default = 0
        Device_8Bits = 1    'use 8 Bit Resolution, Else 16 Bit
        Device_Mono = 2    'use Mono, Else Stereo
        Device_3D = 4    'enable 3D Functionality
        ' If The Bass_Device_3D Flag Is Not Specified When Initilizing Bass,
        ' Then The 3D Flags (Bass_Sample_3D AndAlso Bass_Music_3D) Are Ignored When
        ' Loading/Creating A Sample/Stream/Music.
        'Device_Leavevol = 32		  'leave Volume As It Is
        'Device_Nothread = 128		  'update Buffers Manually (Using Bass_Update)
        Device_Latency = 256    'calculate Device Latency (Bass_Info Struct)
        'Device_Vol1000 = 512		  '0-1000 Volume Range (Else 0-100)
        'Device_Floatdsp = 1024		  'all Dsp Uses 32-Bit Floating-Point Data
        Device_Speakers = 2048    'force Enabling Of Speaker Assignment
    End Enum    ' Bass_Device

    ' BASS_CHANNELINFO types
    Public Enum ChanType As Integer
        BASS_CTYPE_SAMPLE = 1
        BASS_CTYPE_RECORD = 2
        BASS_CTYPE_STREAM = &H10000
        BASS_CTYPE_STREAM_WAV = &H10001
        BASS_CTYPE_STREAM_OGG = &H10002
        BASS_CTYPE_STREAM_MP1 = &H10003
        BASS_CTYPE_STREAM_MP2 = &H10004
        BASS_CTYPE_STREAM_MP3 = &H10005
        BASS_CTYPE_MUSIC_MOD = &H20000
        BASS_CTYPE_MUSIC_MTM = &H20001
        BASS_CTYPE_MUSIC_S3M = &H20002
        BASS_CTYPE_MUSIC_XM = &H20003
        BASS_CTYPE_MUSIC_IT = &H20004
        BASS_CTYPE_MUSIC_MO3 = &H20100     ' mo3 flag
    End Enum    ' ChanType

    Public Enum PlayingMode As Integer
        ' For Bass_ChannelIsActive
        Stopped = 0
        Playing = 1
        Stalled = 2
        Paused = 3
    End Enum    ' PlayingMode

    Public Enum StreamCreateFile
        BASS_CREATEFILE_DEFAULT = 0    ' Default Flags
        BASS_SAMPLE_FLOAT = 256    ' 32-bit floating-point
        BASS_SAMPLE_MONO = 2    ' mono, else stereo
        BASS_SAMPLE_3D = 8    ' 3D functionality enabled
        BASS_SAMPLE_LOOP = 4    ' looped
        BASS_SAMPLE_FX = 128    ' the DX8 effects are enabled
        BASS_MP3_SETPOS = 131072    ' enable pin-point seeking on the MP3/MP2/MP1/OGG
        BASS_STREAM_AUTOFREE = 262144    ' automatically free the stream when it stop/ends
        BASS_STREAM_DECODE = &H200000    ' don't play the stream, only decode (BASS_ChannelGetData)
        BASS_SPEAKER_FRONT = &H1000000    ' front speakers
        BASS_SPEAKER_REAR = &H2000000    ' rear/side speakers
        BASS_SPEAKER_CENLFE = &H3000000    ' center & LFE speakers (5.1)
        BASS_SPEAKER_REAR2 = &H4000000    ' rear center speakers (7.1)
        BASS_SPEAKER_LEFT = &H10000000    ' modifier: left
        BASS_SPEAKER_RIGHT = &H20000000    ' modifier: right
        BASS_SPEAKER_FRONTLEFT = BASS_SPEAKER_FRONT Or BASS_SPEAKER_LEFT
        BASS_SPEAKER_FRONTRIGHT = BASS_SPEAKER_FRONT Or BASS_SPEAKER_RIGHT
        BASS_SPEAKER_REARLEFT = BASS_SPEAKER_REAR Or BASS_SPEAKER_LEFT
        BASS_SPEAKER_REARRIGHT = BASS_SPEAKER_REAR Or BASS_SPEAKER_RIGHT
        BASS_SPEAKER_CENTER = BASS_SPEAKER_CENLFE Or BASS_SPEAKER_LEFT
        BASS_SPEAKER_LFE = BASS_SPEAKER_CENLFE Or BASS_SPEAKER_RIGHT
        BASS_SPEAKER_REAR2LEFT = BASS_SPEAKER_REAR2 Or BASS_SPEAKER_LEFT
        BASS_SPEAKER_REAR2RIGHT = BASS_SPEAKER_REAR2 Or BASS_SPEAKER_RIGHT
    End Enum    ' StreamCreateFile

    Public Enum SetSyncSyncType As Integer
        BASS_SYNC_MESSAGE = &H20000000
        'FLAG: sync at mixtime, else at playtime
        BASS_SYNC_MIXTIME = &H40000000
        ' FLAG: sync only once, else continuously
        BASS_SYNC_ONETIME = &H80000000

        BASS_SYNC_POS = 0
        BASS_SYNC_END = 2
        BASS_SYNC_MUSICINST = 1
        BASS_SYNC_MUSICFX = 3
        BASS_SYNC_META = 4
        BASS_SYNC_SLIDE = 5
        BASS_SYNC_STALL = 6
        BASS_SYNC_DOWNLOAD = 7
    End Enum

#End Region

#Region " BASS Public Structures "

    Public Structure BASS_INFO
        Dim size As Integer     'size of this struct (set this before calling the function)
        Dim flags As Integer    'device capabilities (DSCAPS_xxx flags)
        Dim hwsize As Integer    'size of total device hardware memory
        Dim hwfree As Integer    'size of free device hardware memory
        Dim freesam As Integer    'number of free sample slots in the hardware
        Dim free3d As Integer    'number of free 3D sample slots in the hardware
        Dim minrate As Integer    'min sample rate supported by the hardware
        Dim maxrate As Integer    'max sample rate supported by the hardware
        Dim eax As Integer     'device supports EAX? (always BASSFALSE if BASS_DEVICE_3D was not used)
        Dim minbuf As Integer    'recommended minimum buffer length in ms (requires BASS_DEVICE_LATENCY)
        Dim dsver As Integer    'DirectSound version
        Dim latency As Integer    'delay (in ms) before start of playback (requires BASS_DEVICE_LATENCY)
        Dim initflags As Integer     '"flags" parameter of BASS_Init call
        Dim speakers As Integer    'number of speakers available
        Dim driver As Integer    ' driver
    End Structure    ' BASS_INFO

    Public Structure BASS_RECORDINFO
        Dim size As Integer    ' size of this struct (set this before calling the function)
        Dim flags As Integer    ' device capabilities (DSCCAPS_xxx flags)
        Dim formats As Integer    ' supported standard formats (WAVE_FORMAT_xxx flags)
        Dim inputs As Integer    ' number of inputs
        Dim singlein As Integer    ' BASSTRUE = only 1 input can be set at a time
        Dim driver As Integer    ' driver
    End Structure    ' BASS_RECORDINFO

    '*************************
    '* Sample info structure *
    '*************************
    Public Structure BASS_SAMPLE
        Dim freq As Integer    'default playback rate
        Dim volume As Integer    'default volume (0-100)
        Dim pan As Integer    'default pan (-100=left, 0=middle, 100=right)
        Dim flags As Integer    'BASS_SAMPLE_xxx flags
        Dim Length As Integer    'length (in samples, not bytes)
        Dim max As Integer    ' maximum simultaneous playbacks
        ' The following are the sample's default 3D attributes (if the sample
        ' is 3D, BASS_SAMPLE_3D is in flags) see BASS_ChannelSet3DAttributes
        Dim mode3d As Integer    'BASS_3DMODE_xxx mode
        Dim mindist As Single    'minimum distance
        Dim MAXDIST As Single    'maximum distance
        Dim iangle As Integer    'angle of inside projection cone
        Dim oangle As Integer    'angle of outside projection cone
        Dim outvol As Integer    'delta-volume outside the projection cone
        ' The following are the defaults used if the sample uses the DirectX 7
        ' voice allocation/management features.
        Dim vam As Integer    'voice allocation/management flags (BASS_VAM_xxx)
        Dim priority As Integer    'priority (0=lowest, &Hffffffff=highest)
    End Structure    ' BASS_SAMPLE

    '********************************************************
    '* 3D vector (for 3D positions/velocities/orientations) *
    '********************************************************
    Public Structure BASS_3DVECTOR
        Dim X As Single    '+=right, -=left
        Dim Y As Single    '+=up, -=down
        Dim Z As Single    '+=front, -=behind
    End Structure    ' BASS_3DVECTOR

    Public Structure BASS_FXCHORUS    ' DSFXChorus
        Dim fWetDryMix As Single
        Dim fDepth As Single
        Dim fFeedback As Single
        Dim fFrequency As Single
        Dim lWaveform As Integer    ' 0=triangle, 1=sine
        Dim fDelay As Single
        Dim lPhase As Integer    ' BASS_FX_PHASE_xxx
    End Structure

    Public Structure BASS_FXCOMPRESSOR
        Dim fGain As Single
        Dim fAttack As Single
        Dim fRelease As Single
        Dim fThreshold As Single
        Dim fRatio As Single
        Dim fPredelay As Single
    End Structure    ' DSFXCompressor

    Public Structure BASS_FXDISTORTION
        Dim fGain As Single
        Dim fEdge As Single
        Dim fPostEQCenterFrequency As Single
        Dim fPostEQBandwidth As Single
        Dim fPreLowpassCutoff As Single
    End Structure    ' DSFXDistortion

    Public Structure BASS_FXECHO
        Dim fWetDryMix As Single
        Dim fFeedback As Single
        Dim fLeftDelay As Single
        Dim fRightDelay As Single
        Dim lPanDelay As Integer
    End Structure    ' DSFXEcho

    Public Structure BASS_FXFLANGER
        Dim fWetDryMix As Single
        Dim fDepth As Single
        Dim fFeedback As Single
        Dim fFrequency As Single
        Dim lWaveform As Integer    ' 0=triangle, 1=sine
        Dim fDelay As Single
        Dim lPhase As Integer    ' BASS_FX_PHASE_xxx
    End Structure    ' DSFXFlanger

    Public Structure BASS_FXGARGLE
        Dim dwRateHz As Integer    ' Rate of modulation in hz
        Dim dwWaveShape As Integer    ' 0=triangle, 1=square
    End Structure    ' DSFXGargle

    Public Structure BASS_FXI3DL2REVERB
        Dim lRoom As Integer    ' [-10000, 0]      default: -1000 mB
        Dim lRoomHF As Integer    ' [-10000, 0]      default: 0 mB
        Dim flRoomRolloffFactor As Single    ' [0.0, 10.0]      default: 0.0
        Dim flDecayTime As Single    ' [0.1, 20.0]      default: 1.49s
        Dim flDecayHFRatio As Single    ' [0.1, 2.0]       default: 0.83
        Dim lReflections As Integer    ' [-10000, 1000]   default: -2602 mB
        Dim flReflectionsDelay As Single    ' [0.0, 0.3]       default: 0.007 s
        Dim lReverb As Integer    ' [-10000, 2000]   default: 200 mB
        Dim flReverbDelay As Single    ' [0.0, 0.1]       default: 0.011 s
        Dim flDiffusion As Single    ' [0.0, 100.0]     default: 100.0 %
        Dim flDensity As Single    ' [0.0, 100.0]     default: 100.0 %
        Dim flHFReference As Single    ' [20.0, 20000.0]  default: 5000.0 Hz
    End Structure    ' DSFXI3DL2Reverb

    Public Structure BASS_FXPARAMEQ
        Dim fCenter As Single
        Dim fBandwidth As Single
        Dim fGain As Single
    End Structure    ' DSFXParamEq

    Public Structure BASS_FXREVERB
        Dim fInGain As Single    ' [-96.0,0.0]            default: 0.0 dB
        Dim fReverbMix As Single    ' [-96.0,0.0]            default: 0.0 db
        Dim fReverbTime As Single    ' [0.001,3000.0]         default: 1000.0 ms
        Dim fHighFreqRTRatio As Single    ' [0.001,0.999]          default: 0.001
    End Structure    ' DSFXWavesReverb

    <StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Ansi)> _
     Public Structure GUID    ' used with BASS_Init - use Marshal.StructureToPtr in clsid parameter
        Dim Data1 As Integer
        Dim Data2 As Short
        Dim Data3 As Short
        <MarshalAs(UnmanagedType.ByValArray, SizeConst:=7)> _
        Dim Data4 As Byte()
    End Structure    ' GUID

    Public Structure BASS_CHANNELINFO
        Dim freq As Integer    ' default playback rate
        Dim chan As Integer    ' number of channels... 1=mono, 2=stereo, etc...
        Dim flags As Integer    ' BASS_SAMPLE/STREAM/MUSIC/SPEAKER flags
        Dim chantype As chantype    ' type of channel
    End Structure
    Public Const BASS_ENCODE_NOHEAD As Integer = 1     'do NOT send a WAV header to the encoder
    Public Const BASS_ENCODE_FP_8BIT As Integer = 2    'convert floating-point sample data to 8-bit integer
    Public Const BASS_ENCODE_FP_16BIT As Integer = 4   'convert floating-point sample data to 16-bit integer
    Public Const BASS_ENCODE_FP_24BIT As Integer = 6   'convert floating-point sample data to 24-bit integer
    Public Const BASS_ENCODE_BIGEND As Integer = 16    'big-endian sample data
    Public Const BASS_ENCODE_PAUSE As Integer = 32     'start encording paused


    Declare Function BASS_Encode_Start Lib "bassenc.dll" (ByVal chan As Integer, ByVal cmdline As String, ByVal flags As Integer, ByVal proc As Integer, ByVal user As Integer) As Integer
    Declare Function BASS_Encode_IsActive Lib "bassenc.dll" (ByVal chan As Integer) As Integer
    Declare Function BASS_Encode_Stop Lib "bassenc.dll" (ByVal chan As Integer) As Integer
    Declare Function BASS_Encode_SetPaused Lib "bassenc.dll" (ByVal chan As Integer, ByVal paused As Integer) As Integer
    Declare Function BASS_Encode_Write Lib "bassenc.dll" (ByVal handle As Integer, ByVal buffer As Integer, ByVal length As Integer) As Integer


    Sub ENCODEPROC(ByVal channel As Integer, ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer)

        'CALLBACK FUNCTION !!!

        ' Encoding callback function.
        ' channel: The channel handle
        ' buffer : Buffer containing the encoded data
        ' length : Number of bytes
        ' user   : The 'user' parameter value given when calling BASS_EncodeStart

    End Sub

#End Region

#Region " BASS API Declarations "

#Region " Standard Win32 DLL Calls "

    Private Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (ByVal pDst As IntPtr, _
     ByVal pSrc As String, _
     ByVal ByteLen As Long)

    Private Declare Function lstrlen Lib "kernel32" Alias "lstrlenA" (ByVal lpString As Integer) As Integer

#End Region

#Region " BASS General Declarations "

    Declare Function BASS_SetConfig Lib "bass.dll" (ByVal opt As BassConfigOptions, ByVal value As Integer) As Integer
    Declare Function BASS_GetConfig Lib "bass.dll" (ByVal opt As BassConfigOptions) As Integer

    ' Retrieve the version number of BASS that is loaded. RETURN : The BASS version (LOWORD.HIWORD)
    Declare Function BASS_GetVersion Lib "bass.dll" () As Integer

    ' Gets a pointer to the text description of a device. This function can be used to enumerate the available devices.
    ' devnum: The device (0=First)
    ' RETURN : The text description of the device (NULL=error)
    ' NOTE: use BASS_GetDeviceDestriptionString to get the text description
    Declare Function BASS_GetDeviceDescription Lib "bass.dll" (ByVal devnum As Integer) As IntPtr

    ' Set the amount that BASS mixes ahead new musics/streams.
    ' Changing this setting does not affect musics/streams
    ' that have already been loaded/created. Increasing the
    ' buffer length, decreases the chance of the sound
    ' possibly breaking-up on slower computers, but also
    ' requires more memory. The default length is 0.5 secs.
    ' length : The buffer length in seconds
    ' RETURN : The actual new buffer length
    Declare Function BASS_SetBufferLength Lib "bass.dll" (ByVal Length As Single) As Single

    ' Set the global music/sample/stream volume levels.
    ' musvol : MOD music global volume level (0-100, -1=leave current)
    ' samvol : Sample global volume level (0-100, -1=leave current)
    ' strvol : Stream global volume level (0-100, -1=leave current)
    Declare Sub BASS_SetGlobalVolumes Lib "bass.dll" (ByVal musvol As Integer, ByVal samvol As Integer, ByVal strvol As Integer)

    ' Retrive the global music/sample/stream volume levels.
    ' musvol : MOD music global volume level (NULL=don't retrieve it)
    ' samvol : Sample global volume level (NULL=don't retrieve it)
    ' strvol : Stream global volume level (NULL=don't retrieve it)
    Declare Sub BASS_GetGlobalVolumes Lib "bass.dll" (ByRef musvol As Integer, ByRef samvol As Integer, ByRef strvol As Integer)

    ' Make the volume/panning values translate to a logarithmic curve,
    ' orelse a linear "curve" (the default)
    ' volume :   volume curve(BASSFALSE = linear, BASSTRUE = Log)
    ' pan    : panning curve (BASSFALSE=linear, BASSTRUE=log) */
    Declare Sub BASS_SetLogCurves Lib "bass.dll" (ByVal volume As Integer, ByVal pan As Integer)

    ' Set the 3D algorithm for software mixed 3D channels (does not affect
    ' hardware mixed channels). Changing the mode only affects subsequently
    ' created orelse loaded samples/streams/musics, not those that already exist.
    ' Requires DirectX 7 orelse above.
    ' algo : algorithm flag (BASS_3DALG_xxx)
    Declare Sub BASS_Set3DAlgorithm Lib "bass.dll" (ByVal algo As Integer)

    ' Get the BASS_ERROR_xxx error code. Use this function to get the reason for an error.
    Declare Function BASS_ErrorGetCode Lib "bass.dll" () As Bass_ErrorCodes

    ' Set the class identifier of the object to create, that will be used
    ' to initialize DirectSound.
    ' clsid  : Class identifier of the object to create (NULL=use default)
    Declare Sub BASS_SetCLSID Lib "bass.dll" (ByRef clsid As GUID)

    ' Initialize the digital output. This must be called before all
    ' following BASS functions (except CD and recording functions).
    ' The volume is initially set to 100 (the maximum),
    ' use BASS_SetVolume() to adjust it.
    ' device : Device to use (0=first, -1=default, -2=no sound)
    ' freq   : Output sample rate
    ' flags:     BASS_DEVICE_xxx flags (optional HIWORD=update period)
    ' win:       Owner window (0=current foreground window)

    ' NOTE: The "no sound" device (device=-2), allows loading
    ' and "playing" of MOD musics only (all sample/stream
    ' functions and most other functions fail). This is so
    ' that you can still use the MOD musics as synchronizers
    ' when there is no soundcard present. When using device -2,
    ' you should still set the other arguments as you would do
    ' normally.
    Declare Function BASS_Init Lib "bass.dll" (ByVal device As Integer, ByVal freq As Integer, ByVal flags As Bass_Device, ByVal win As Integer, ByVal guid As Integer) As BassBool

    ' Free all resources used by the digital output, including  all musics and samples.
    Declare Function BASS_SetDevice Lib "bass.dll" (ByVal device As Integer) As BassBool
    Declare Function BASS_GetDevice Lib "bass.dll" () As Integer
    Declare Function BASS_Free Lib "bass.dll" () As BassBool

    ' Retrieve a pointer to a DirectSound interface. This can be used by
    ' advanced users to "plugin" external functionality.
    ' object : The interface to retrieve (BASS_OBJECT_xxx)
    ' RETURN : A pointer to the requested interface (NULL=error)
    Declare Function BASS_GetDSoundObject Lib "bass.dll" (ByVal object_Renamed As Integer) As Integer

    ' Retrieve some information on the device being used.
    ' info   : Pointer to store info at
    Declare Function BASS_GetInfo Lib "bass.dll" (ByRef info As BASS_INFO) As BassBool

    ' Update the HMUSIC/HSTREAM channel buffers.
    Declare Function BASS_Update Lib "bass.dll" () As BassBool

    ' Get the current CPU usage of BASS. This includes the time taken to mix
    ' the MOD musics and sample streams, and also the time taken by any user
    ' DSP functions. It does not include plain sample mixing which is done by
    ' the output device (hardware accelerated) orelse DirectSound (emulated). Audio
    ' CD playback requires no CPU usage.
    ' RETURN : The CPU usage percentage (floating-point)
    Declare Function BASS_GetCPU Lib "bass.dll" () As Single

    ' Start the digital output.
    Declare Function BASS_Start Lib "bass.dll" () As BassBool

    ' Stop the digital output, stopping all musics/samples/streams.
    Declare Function BASS_Stop Lib "bass.dll" () As BassBool

    ' Stop the digital output, pausing all musics/samples/
    ' streams. Use BASS_Start to resume the digital output.
    Declare Function BASS_Pause Lib "bass.dll" () As BassBool

    ' Set the digital output master volume.
    ' volume : Desired volume level (0-100)
    Declare Function BASS_SetVolume Lib "bass.dll" (ByVal volume As Integer) As BassBool

    ' Get the digital output master volume.
    ' RETURN : The volume level (0-100, -1=error)
    Declare Function BASS_GetVolume Lib "bass.dll" () As Integer

    ' Set the factors that affect the calculations of 3D sound.
    ' distf  : Distance factor (0.0-10.0, 1.0=use meters, 0.3=use feet, <0.0=leave current)
    '          By default BASS measures distances in meters, you can change this
    '          setting if you are using a different unit of measurement.
    ' roolf  : Rolloff factor, how fast the sound quietens with distance
    '          (0.0=no rolloff, 1.0=real world, 2.0=2x real... 10.0=max, <0.0=leave current)
    ' doppf  : Doppler factor (0.0=no doppler, 1.0=real world, 2.0=2x real... 10.0=max, <0.0=leave current)
    '          The doppler effect is the way a sound appears to change frequency when it is
    '          moving towards orelse away from you. The listener and sound velocity settings are
    '          used to calculate this effect, this "doppf" value can be used to lessen or
    '          exaggerate the effect.
    Declare Function BASS_Set3DFactors Lib "bass.dll" (ByVal distf As Single, ByVal rollf As Single, ByVal doppf As Single) As BassBool

    ' Get the factors that affect the calculations of 3D sound.
    ' distf  : Distance factor (NULL=don't get it)
    ' roolf  : Rolloff factor (NULL=don't get it)
    ' doppf  : Doppler factor (NULL=don't get it)
    Declare Function BASS_Get3DFactors Lib "bass.dll" (ByRef distf As Single, ByRef rollf As Single, ByRef doppf As Single) As BassBool

    ' Set the position/velocity/orientation of the listener (ie. the player/viewer).
    ' pos    : Position of the listener (NULL=leave current)
    ' vel    : listener 's velocity, used to calculate doppler effect (NULL=leave current)
    ' front  : Direction that listener's front is pointing (NULL=leave current)
    ' top    : Direction that listener's top is pointing (NULL=leave current)
    ' NOTE   : front & top must both be set in a single call
    Declare Function BASS_Set3DPosition Lib "bass.dll" (ByRef pos As Integer, ByRef vel As Integer, ByRef front As Integer, ByRef top As Integer) As BassBool

    ' Get the position/velocity/orientation of the listener.
    ' pos    : Position of the listener (NULL=don't get it)
    ' vel    : listener 's velocity (NULL=don't get it)
    ' front  : Direction that listener's front is pointing (NULL=don't get it)
    ' top    : Direction that listener's top is pointing (NULL=don't get it)
    ' NOTE   : front & top must both be retrieved in a single call
    Declare Function BASS_Get3DPosition Lib "bass.dll" (ByRef pos As Integer, ByRef vel As Integer, ByRef front As Integer, ByRef top As Integer) As BassBool

    ' Apply changes made to the 3D system. This must be called to apply any changes
    ' made with BASS_Set3DFactors, BASS_Set3DPosition, BASS_ChannelSet3DAttributes or
    ' BASS_ChannelSet3DPosition. It improves performance to have DirectSound do all the
    ' required recalculating at the same time like this, rather than recalculating after
    ' every little change is made. NOTE: This is automatically called when starting a 3D
    ' sample with BASS_SamplePlay3D/Ex.
    Declare Function BASS_Apply3D Lib "bass.dll" () As BassBool

    ' Set the type of EAX environment and it's parameters. Obviously, EAX functions
    ' have no effect if no EAX supporting device (ie. SB Live) is used.
    ' env    : Reverb environment (EAX_ENVIRONMENT_xxx, -1=leave current)
    ' vol    : Volume of the reverb (0.0=off, 1.0=max, <0.0=leave current)
    ' decay  : Time in seconds it takes the reverb to diminish by 60dB (0.1-20.0, <0.0=leave current)
    ' damp   : The damping, high orelse low frequencies decay faster (0.0=high decays quickest,
    '          1.0=low/high decay equally, 2.0=low decays quickest, <0.0=leave current)
    Declare Function BASS_SetEAXParameters Lib "bass.dll" (ByVal env As Integer, ByVal vol As Single, ByVal decay As Single, ByVal damp As Single) As BassBool

    ' Get the current EAX parameters.
    ' env    : Reverb environment (NULL=don't get it)
    ' vol    : Reverb volume (NULL=don't get it)
    ' decay  : Decay duration (NULL=don't get it)
    ' damp   : The damping (NULL=don't get it)
    Declare Function BASS_GetEAXParameters Lib "bass.dll" (ByRef env As Integer, ByRef vol As Single, ByRef decay As Single, ByRef damp As Single) As BassBool

#End Region

#Region " BASS Music Declarations "

    ' Load a music (MO3/XM/MOD/S3M/IT/MTM). The amplification and pan
    ' seperation are initially set to 50, use BASS_MusicSetAmplify()
    ' and BASS_MusicSetPanSep() to adjust them.
    ' mem    : BASSTRUE = Load music from memory
    ' f      : Filename (mem=BASSFALSE) orelse memory location (mem=BASSTRUE)
    ' offset : File offset to load the music from (only used if mem=BASSFALSE)
    ' length : Data length (only used if mem=BASSFALSE, 0=use to end of file)
    ' flags  : BASS_MUSIC_xxx flags
    ' RETURN : The loaded music's handle (NULL=error)
    Declare Function BASS_MusicLoad Lib "bass.dll" (ByVal mem As BassBool, ByVal f As String, ByVal offset As Integer, ByVal Length As Integer, ByVal flags As Integer, ByVal freq As Integer) As Integer

    '  Free a music's resources. handle =  Music handle
    Declare Sub BASS_MusicFree Lib "bass.dll" (ByVal handle As Integer)

    ' Retrieves a pointer to music's name.
    ' handle : Music handle
    ' RETURN : Pointer to the Music 's name (NULL=error)
    ' NOTE: Use the function BASS_MusicGetNameString to get the actual name as a string.
    Declare Function BASS_MusicGetName Lib "bass.dll" (ByVal handle As Integer) As IntPtr

    ' Retrieves the length of a music in patterns (how many "orders" there are)
    ' orelse in output bytes (requires BASS_MUSIC_CALCLEN was used with BASS_MusicLoad).
    ' handle : Music handle
    ' playlen: BASSTRUE=get the playback length, BASSFALSE=get the pattern length
    ' RETURN : The length of the music (-1=error)
    Declare Function BASS_MusicGetLength Lib "bass.dll" (ByVal handle As Integer, ByVal playlen As Integer) As Integer

    ' Pre-buffer initial sample data ready for playback.
    ' handle : Handle of music
    Declare Function BASS_MusicPreBuf Lib "bass.dll" (ByVal handle As Integer) As BassBool

    ' Play a music. Playback continues from where it was last stopped/paused.
    ' Multiple musics may be played simultaneously.
    ' handle : Handle of music to play
    Declare Function BASS_MusicPlay Lib "bass.dll" (ByVal handle As Integer) As BassBool

    ' Play a music, specifying start position and playback flags.
    ' handle : Handle of music to play
    ' pos    : Position to start playback from, LOWORD=order HIWORD=row
    ' flags  : BASS_MUSIC_xxx flags. These flags overwrite the defaults
    '          specified when the music was loaded. (-1=use current flags)
    ' reset_Renamed  : BASSTRUE = Stop all current playing notes and reset bpm/etc...
    Declare Function BASS_MusicPlayEx Lib "bass.dll" (ByVal handle As Integer, ByVal pos As Integer, ByVal flags As Integer, ByVal reset_Renamed As Integer) As BassBool

    ' Set a music's amplification level.
    ' handle:    Music handle
    ' amp:       Amplification Level(0 - 100)
    Declare Function BASS_MusicSetAmplify Lib "bass.dll" (ByVal handle As Integer, ByRef amp As Integer) As BassBool

    ' Set a music's pan seperation.
    ' handle:    Music handle
    ' pan:       pan seperation(0 - 100, 50 = linear)
    Declare Function BASS_MusicSetPanSep Lib "bass.dll" (ByVal handle As Integer, ByRef pan As Integer) As BassBool

    ' Set a music's "GetPosition" scaler
    ' When you call BASS_ChannelGetPosition, the "row" (HIWORD) will be
    ' scaled by this value. By using a higher scaler, you can get a more
    ' precise position indication.
    ' handle:    Music handle
    ' Scale: The scaler(1 - 256)
    Declare Function BASS_MusicSetPositionScaler Lib "bass.dll" (ByVal handle As Integer, ByVal pscale As Integer) As BassBool

    ' Set the volume level of a channel in a music
    ' handle : Music handle
    ' channel: Channel number (0=first)
    ' volume : Volume level (0-100)
    Declare Function BASS_MusicSetVolume Lib "bass.dll" (ByVal handle As Integer, ByVal channel As Integer, ByVal volume As Integer) As BassBool

    ' Get the volume level of a channel in a music
    ' handle : Music handle
    ' channel: Channel number (0=first)
    ' RETURN : The channel's volume (-1=error)
    Declare Function BASS_MusicGetVolume Lib "bass.dll" (ByVal handle As Integer, ByVal channel As Integer) As Integer


#End Region

#Region " BASS Sample Declarations "

    ' Load a WAV/MP3/MP2/MP1 sample. If you're loading a sample with 3D
    ' functionality, then you should use BASS_GetInfo and BASS_SetInfo to set
    ' the default 3D parameters. You can also use these two functions to set
    ' the sample's default frequency/volume/pan/looping.
    ' mem    : BASSTRUE = Load sample from memory
    ' f      : Filename (mem=BASSFALSE) orelse memory location (mem=BASSTRUE)
    ' offset : File offset to load the sample from (only used if mem=BASSFALSE)
    ' length : Data length (only used if mem=BASSFALSE, 0=use to end of file)
    ' max    : Maximum number of simultaneous playbacks (1-65535)
    ' flags  : BASS_SAMPLE_xxx flags (only the LOOP/3D/SOFTWARE/DEFER/MUTEMAX/OVER_xxx flags are used)
    ' RETURN : The loaded sample's handle (NULL=error)
    Declare Function BASS_SampleLoad Lib "bass.dll" (ByVal mem As BassBool, ByVal f As String, ByVal offset As Integer, ByVal Length As Integer, ByVal max As Integer, ByVal flags As Integer) As Integer

    ' Create a sample. This function allows you to generate custom samples, or
    ' load samples that are not in the WAV format. A pointer is returned to the
    ' memory location at which you should write the sample's data. After writing
    ' the data, call BASS_SampleCreateDone to get the new sample's handle.
    ' length:    The sample 's length (in samples, NOT bytes)
    ' freq   : default sample rate
    ' max    : Maximum number of simultaneous playbacks (1-65535)
    ' flags:     BASS_SAMPLE_xxx flags
    ' RETURN : Memory location to write the sample's data (NULL=error)
    Declare Function BASS_SampleCreate Lib "bass.dll" (ByVal Length As Integer, ByVal freq As Integer, ByVal max As Integer, ByVal flags As Integer) As Integer

    ' Finished creating a new sample.
    ' Return: The New sample 's handle (NULL=error)
    Declare Function BASS_SampleCreateDone Lib "bass.dll" () As Integer

    ' Free a sample's resources.
    ' handle:    Sample handle
    Declare Sub BASS_SampleFree Lib "bass.dll" (ByVal handle As Integer)

    ' Retrieve a sample's current default attributes.
    ' handle:    Sample handle
    ' info   : Pointer to store sample info
    Declare Function BASS_SampleGetInfo Lib "bass.dll" (ByVal handle As Integer, ByRef info As BASS_SAMPLE) As BassBool

    'Set a sample's default attributes.
    ' handle:    Sample handle
    ' info   : Sample info, only the freq/volume/pan/3D attributes and
    '          looping/override method flags are used
    Declare Function BASS_SampleSetInfo Lib "bass.dll" (ByVal handle As Integer, ByRef info As BASS_SAMPLE) As BassBool

    ' Play a sample, using the sample's default attributes.
    ' handle : Handle of sample to play
    ' RETURN : Handle of channel used to play the sample (NULL=error)
    Declare Function BASS_SamplePlay Lib "bass.dll" (ByVal handle As Integer) As Integer

    ' Play a sample, using specified attributes.
    ' handle : Handle of sample to play
    ' start  : Playback start position (in samples, not bytes)
    ' freq:      Playback Rate(-1 = Default)
    ' volume : Volume (-1=default, 0=silent, 100=max)
    ' pan:       pan position(-101 = Default, -100 = Left, 0 = middle, 100 = Right)
    ' loop   : 1 = Loop sample (-1=default)
    ' RETURN : Handle of channel used to play the sample (NULL=error)
    Declare Function BASS_SamplePlayEx Lib "bass.dll" (ByVal handle As Integer, ByVal start As Integer, ByVal freq As Integer, ByVal volume As Integer, ByVal pan As Integer, ByVal pLoop As Integer) As Integer

    ' Play a 3D sample, setting it's 3D position, orientation and velocity.
    ' handle : Handle of sample to play
    ' pos    : position of the sound (NULL = x/y/z=0.0)
    ' orient : orientation of the sound, this is irrelevant if it's an
    '          omnidirectional sound source (NULL = x/y/z=0.0)
    ' vel    : velocity of the sound (NULL = x/y/z=0.0)
    ' RETURN : Handle of channel used to play the sample (NULL=error)
    Declare Function BASS_SamplePlay3D Lib "bass.dll" (ByVal handle As Integer, ByRef pos As Integer, ByRef orient As Integer, ByRef vel As Integer) As Integer

    ' Play a 3D sample, using specified attributes.
    ' handle : Handle of sample to play
    ' pos    : position of the sound (NULL = x/y/z=0.0)
    ' orient : orientation of the sound, this is irrelevant if it's an
    '          omnidirectional sound source (NULL = x/y/z=0.0)
    ' vel    : velocity of the sound (NULL = x/y/z=0.0)
    ' start  : Playback start position (in samples, not bytes)
    ' freq:      Playback Rate(-1 = Default)
    ' volume : Volume (-1=default, 0=silent, 100=max)
    ' loop   : 1 = Loop sample (-1=default)
    ' RETURN : Handle of channel used to play the sample (NULL=error)
    Declare Function BASS_SamplePlay3DEx Lib "bass.dll" (ByVal handle As Integer, ByRef pos As Integer, ByRef orient As Integer, ByRef vel As Integer, ByVal start As Integer, ByVal freq As Integer, ByVal volume As Integer, ByVal pLoop As Integer) As Integer

    ' Stops all instances of a sample. For example, if a sample is playing
    ' simultaneously 3 times, calling this function will stop all 3 of them,
    ' which is obviously simpler than calling BASS_ChannelStop() 3 times.
    ' handle : Handle of sample to stop
    Declare Function BASS_SampleStop Lib "bass.dll" (ByVal handle As Integer) As BassBool

#End Region

#Region " BASS Stream Declarations "
    Declare Function BASS_AAC_StreamCreateFile Lib "bass_aac.dll" (ByVal file As String, ByVal flags As Long) As Integer
    Declare Function BASS_MP4_StreamCreateFile Lib "bass_aac.dll" (ByVal file As String, ByVal flags As Long) As Integer

    ' Create a user sample stream.
    ' freq   : Stream playback rate
    ' flags  : BASS_SAMPLE_xxx flags (only the 8BITS/MONO/3D flags are used)
    ' proc   : User defined stream writing function pass using "AddressOf STREAMPROC"
    ' user   : The 'user' value passed to the callback function
    ' RETURN : The created stream's handle (NULL=error)
    Declare Function BASS_StreamCreate Lib "bass.dll" (ByVal freq As Integer, ByVal chans As Integer, ByVal flags As Integer, ByVal proc As STREAMPROC_Handler, ByVal user As Integer) As Integer

    ' Create a sample stream from an MP3/MP2/MP1/OGG orelse WAV file.
    ' mem    : BASSTRUE = Stream file from memory
    ' f      : Filename (mem=BASSFALSE) orelse memory location (mem=BASSTRUE)
    ' offset : File offset of the stream data
    ' length : File length (0=use whole file if mem=BASSFALSE)
    ' flags  : Flags
    ' RETURN : The created stream's handle (NULL=error)
    Declare Function BASS_StreamCreateFile Lib "bass.dll" (ByVal mem As BassBool, ByVal f As String, ByVal offset As Integer, ByVal Length As Integer, ByVal flags As StreamCreateFile) As Integer

    ' Creates a sample stream from an MP3, MP2, MP1, OGG orelse WAV file via a user callback function
    ' buffered: BASSTRUE = Buffered file system
    ' flags   : BASS_SAMPLE_xxx flags (only the 8BITS/MONO/3D flags are used)
    ' proc    : User defined stream writing function pass using "AddressOf STREAMPROC"
    ' user    : The 'user' value passed to the callback function
    ' RETURN  : The created stream's handle (NULL=error)
    Declare Function BASS_StreamCreateFileUser Lib "bass.dll" (ByVal buffered As BassBool, ByVal flags As Integer, ByVal proc As STREAMFILEPROC_Handler, ByVal user As Integer) As Integer

    ' Create a sample stream from an MP3/MP2/MP1/OGG orelse WAV file on the internet.
    ' url    : The URL (beginning with "http://" orelse "ftp://")
    ' offset : File offset of start streaming from
    ' flags  : Flags
    ' proc   : The callback function. Use AddressOf
    ' user   : The 'user' value passed to the callback function
    ' RETURN : The created stream's handle (NULL=error)
    Declare Function BASS_StreamCreateURL Lib "bass.dll" (ByVal url As String, ByVal offset As Integer, ByVal flags As Integer, ByVal proc As DOWNLOADPROC_Handler, ByVal user As Integer) As Integer

    ' Free a sample stream's resources.
    ' stream:    stream handle
    Declare Sub BASS_StreamFree Lib "bass.dll" (ByVal handle As Integer)

    ' Retrieves the playback length (in bytes) of a file stream. It's not always
    ' possible to 100% accurately guess the length of a stream, so the length returned
    ' may be only an approximation when using some WAV codecs.
    ' handle :  Stream handle
    ' RETURN : The length (0=streaming in blocks, -1=error)
    Declare Function BASS_StreamGetLength Lib "bass.dll" (ByVal handle As Integer) As Long

    ' Retrieves the requested tags/headers, if available.
    ' handle : Stream handle
    ' tags   : A BASS_TAG_xxx flag
    ' RETURN : Pointer to the tags (NULL=error)
    Declare Function BASS_StreamGetTags Lib "bass.dll" (ByVal handle As Integer, ByVal tags As Bass_Tag) As Integer

    ' Pre-buffer initial sample data ready for playback.
    ' handle : Handle of stream
    Declare Function BASS_StreamPreBuf Lib "bass.dll" (ByVal handle As Integer) As BassBool

    ' Play a sample stream, optionally flushing the buffer first.
    ' handle : Handle of stream to play
    ' flush  : Flush buffer contents. If you stop a stream and then want to
    '          continue it from where it stopped, don't flush it. Flushing
    '          a file stream causes it to restart from the beginning.
    ' flags  : BASS_SAMPLE_xxx flags (only affects file streams, and only the
    '          LOOP flag is used)
    Declare Function BASS_StreamPlay Lib "bass.dll" (ByVal handle As Integer, ByVal flush As BassBool, ByVal flags As Integer) As BassBool

    ' Retrieves the file position of the decoding, the download (if streaming from
    ' the internet), orelse the end (total length). Obviously only works with file streams.
    ' handle : Stream handle
    ' mode   : The position to retrieve (0=decoding, 1=download, 2=end)
    ' RETURN : The position (-1=error)
    Declare Function BASS_StreamGetFilePosition Lib "bass.dll" (ByVal handle As Integer, ByVal mode As Integer) As Integer

#End Region

#Region " BASS Recording Declarations "

    ' Initialize a recording device.
    ' device : Device to use (0=first, -1=default)
    Declare Function BASS_RecordInit Lib "bass.dll" (ByVal device As Integer) As BassBool

    ' Free all resources used by the recording device.
    Declare Function BASS_RecordFree Lib "bass.dll" () As BassBool

    ' Retrieve some information on the recording device being used.
    ' info   : Pointer to store info at
    Declare Sub BASS_RecordGetInfo Lib "bass.dll" (ByRef info As BASS_RECORDINFO)

    ' Get the text description of a recording input.
    ' inputn : Input number (0=first)
    ' RETURN : The text description (0=error)
    Declare Function BASS_RecordGetInputName Lib "bass.dll" (ByVal inputn As Integer) As Integer

    ' Adjust the setting of a recording input.
    ' inputn : Input number (0=first)
    ' setting: BASS_INPUT flags (if BASS_INPUT_LEVEL used, LOWORD=volume)
    Declare Function BASS_RecordSetInput Lib "bass.dll" (ByVal inputn As Integer, ByVal setting As Integer) As BassBool

    ' Retrieve the setting of a recording input.
    ' inputn : Input number (0=first)
    ' RETURN : The setting (LOWORD=volume, with BASS_INPUT_OFF flag if off, -1=error)
    Declare Function BASS_RecordGetInput Lib "bass.dll" (ByVal inputn As Integer) As Integer

    ' Start recording.
    ' freq   : Sampling rate
    ' flags  : BASS_SAMPLE_8BITS/MONO flags (optional HIWORD=update period)
    ' proc   : User defined function to receive the recorded data
    ' user   : The 'user' value passed to the callback function
    ' RETURN : The recording handle
    Declare Function BASS_RecordStart Lib "bass.dll" (ByVal freq As Integer, ByVal flags As Integer, ByVal proc As RECORDPROC_Handler, ByVal user As Integer) As Integer

    ' Get the text description of a recording device. This function can be
    ' used to enumerate the available devices.
    ' devnum : The device (0=first)
    ' RETURN : The text description of the device (NULL=error)
    Declare Function BASS_RecordGetDeviceDescription Lib "bass.dll" (ByVal devnum As Integer) As IntPtr

    ' Sets the recording device number
    Declare Function BASS_RecordSetDevice Lib "bass.dll" (ByVal device As Integer) As BassBool

    ' Gets the recording device number
    Declare Function BASS_RecordGetDevice Lib "bass.dll" () As Integer


#End Region

#Region " BASS Channel Declarations "

    '*******************************************************************************
    '* A "channel" can be a playing sample (HCHANNEL), a MOD music (HMUSIC), a     *
    '* sample stream (HSTREAM), the CD (CDCHANNEL), orelse recording (RECORDCHAN). The *
    '* following functions can be used with one orelse more of these channel types.    *
    '*******************************************************************************

    ' Translate a byte position into time (seconds)
    ' handle : Handle of channel (HCHANNEL/HMUSIC/HSTREAM, also HSAMPLE)
    ' pos    : The position to translate
    ' RETURN : The millisecond position (-1=error)
    Declare Function BASS_ChannelBytes2Seconds Lib "bass.dll" (ByVal handle As Integer, ByVal pos As Long) As Single

    ' Translate a time (seconds) position into bytes
    ' handle : Handle of channel (HCHANNEL/HMUSIC/HSTREAM, also HSAMPLE)
    ' pos    : The position to translate
    ' RETURN : The byte position (-1=error)
    Declare Function BASS_ChannelSeconds2Bytes Lib "bass.dll" (ByVal handle As Integer, ByVal pos As Single) As Long

    ' Translate a byte position into time (seconds)
    ' handle : Handle of channel (HCHANNEL/HMUSIC/HSTREAM, also HSAMPLE)
    ' RETURN : The device number
    Declare Function BASS_ChannelGetDevice Lib "bass.dll" (ByVal handle As Integer) As Integer

    ' Check if a channel is active (playing) orelse stalled.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL/RECORDCHAN)
    ' RETURN : One of the BASS_ACTIVE_xxx values
    Declare Function BASS_ChannelIsActive Lib "bass.dll" (ByVal handle As Integer) As PlayingMode

    ' Get some info about a channel.
    ' handle:    channel handle(HCHANNEL / HMUSIC / HSTREAM)
    ' RETURN : BASS_SAMPLE_xxx flags (-1=error)
    Declare Function BASS_ChannelGetFlags Lib "bass.dll" (ByVal handle As Integer) As Integer

    ' Get channel information
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL/RECORDCHAN)
    ' info : Pointer to a bass channel info structure
    Declare Function BASS_ChannelGetInfo Lib "bass.dll" (ByVal handle As Integer, ByRef info As BASS_CHANNELINFO) As Bass.BassBool

    ' Stop a channel.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL/RECORDCHAN)
    Declare Function BASS_ChannelStop Lib "bass.dll" (ByVal handle As Integer) As BassBool

    ' Pause a channel.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL/RECORDCHAN)
    Declare Function BASS_ChannelPause Lib "bass.dll" (ByVal handle As Integer) As BassBool
    Declare Function BASS_ChannelPlay Lib "bass.dll" (ByVal handle As Integer, Optional ByVal restart As Integer = 0) As Integer
    ' Resume a paused channel.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL/RECORDCHAN)
    Declare Function BASS_channelresume Lib "bass.dll" (ByVal handle As Integer) As BassBool

    ' Update a channel's attributes. The actual setting may not be exactly
    ' as specified, depending on the accuracy of the device and drivers.
    ' NOTE: Only the volume can be adjusted for the CD "channel", but not all
    ' soundcards allow controlling of the CD volume level.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL)
    ' freq   : Playback rate (-1=leave current)
    ' volume : Volume (-1=leave current, 0=silent, 100=max)
    ' pan    : pan position(-101 = current, -100 = Left, 0 = middle, 100 = Right)
    '          panning has no effect on 3D channels
    Declare Function BASS_ChannelSetAttributes Lib "bass.dll" (ByVal handle As Integer, ByVal freq As Integer, ByVal volume As Integer, ByVal pan As Integer) As BassBool

    ' Retrieve a channel's attributes. Only the volume is available for
    ' the CD "channel" (if allowed by the soundcard/drivers).
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL)
    ' freq   : Pointer to store playback rate (NULL=don't retrieve it)
    ' volume : Pointer to store volume (NULL=don't retrieve it)
    ' pan    : Pointer to store pan position (NULL=don't retrieve it)
    Declare Function BASS_ChannelGetAttributes Lib "bass.dll" (ByVal handle As Integer, ByRef freq As Integer, ByRef volume As Integer, ByRef pan As Integer) As BassBool

    ' Slide a channel's attributes over a period of time.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL)
    ' freq   : Playback rate (100-100000, -1=leave current)
    ' volume : Volume (-1=leave current, 0=silent, 100=max, -2 = fadeout and stop channel)
    ' pan    : Pan position (-101=current, -100=left, 0=middle, 100=right)
    ' time   : Duration of the slide (in milliseconds)
    Declare Function BASS_ChannelSlideAttributes Lib "bass.dll" (ByVal handle As Integer, ByVal freq As Integer, ByVal volume As Integer, ByVal pan As Integer, ByVal time As Integer) As BassBool

    ' Check if a channel's attributes (freq/vol/pan) are sliding.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL)
    ' RETURN : Combination of the BASS_SLIDE_xxx flags
    Declare Function BASS_ChannelIsSliding Lib "bass.dll" (ByVal handle As Integer) As Integer

    ' Set a channel's 3D attributes.
    ' handle : channel handle(HCHANNEL / HSTREAM / HMUSIC)
    ' mode   : BASS_3DMODE_xxx mode (-1=leave current setting)
    ' min    : minimum distance, volume stops increasing within this distance (<0.0=leave current)
    ' max    : maximum distance, volume stops decreasing past this distance (<0.0=leave current)
    ' iangle : angle of inside projection cone in degrees (360=omnidirectional, -1=leave current)
    ' oangle : angle of outside projection cone in degrees (-1=leave current)
    '          NOTE: iangle & oangle must both be set in a single call
    ' outvol : delta-volume outside the projection cone (0=silent, 100=same as inside)
    ' The iangle/oangle angles decide how wide the sound is projected around the
    ' orientation angle. Within the inside angle the volume level is the channel
    ' level as set with BASS_ChannelSetAttributes, from the inside to the outside
    ' angles the volume gradually changes by the "outvol" setting.
    Declare Function BASS_ChannelSet3DAttributes Lib "bass.dll" (ByVal handle As Integer, ByVal mode As Integer, ByVal min As Single, ByVal max As Single, ByVal iangle As Integer, ByVal oangle As Integer, ByVal outvol As Integer) As BassBool

    ' Retrieve a channel's 3D attributes.
    ' handle : channel handle(HCHANNEL / HSTREAM / HMUSIC)
    ' mode   : BASS_3DMODE_xxx mode (NULL=don't retrieve it)
    ' min    : minumum distance (NULL=don't retrieve it)
    ' max    : maximum distance (NULL=don't retrieve it)
    ' iangle : angle of inside projection cone (NULL=don't retrieve it)
    ' oangle : angle of outside projection cone (NULL=don't retrieve it)
    '          NOTE: iangle & oangle must both be retrieved in a single call
    ' outvol : delta-volume outside the projection cone (NULL=don't retrieve it)
    Declare Function BASS_ChannelGet3DAttributes Lib "bass.dll" (ByVal handle As Integer, ByRef mode As Integer, ByRef min As Single, ByRef max As Single, ByRef iangle As Integer, ByRef oangle As Integer, ByRef outvol As Integer) As BassBool

    ' Update a channel's 3D position, orientation and velocity. The velocity
    ' is only used to calculate the doppler effect.
    ' handle : channel handle(HCHANNEL / HSTREAM / HMUSIC)
    ' pos    : position of the sound (NULL=leave current)
    ' orient : orientation of the sound, this is irrelevant if it's an
    '          omnidirectional sound source (NULL=leave current)
    ' vel    : velocity of the sound (NULL=leave current)
    Declare Function BASS_ChannelSet3DPosition Lib "bass.dll" (ByVal handle As Integer, ByRef pos As Integer, ByRef orient As Integer, ByRef vel As Integer) As BassBool

    ' Retrieve a channel's current 3D position, orientation and velocity.
    ' handle : channel handle(HCHANNEL / HSTREAM / HMUSIC)
    ' pos    : position of the sound (NULL=don't retrieve it)
    ' orient : orientation of the sound, this is irrelevant if it's an
    '          omnidirectional sound source (NULL=don't retrieve it)
    ' vel    : velocity of the sound (NULL=don't retrieve it)
    Declare Function BASS_ChannelGet3DPosition Lib "bass.dll" (ByVal handle As Integer, ByRef pos As Integer, ByRef orient As Integer, ByRef vel As Integer) As BassBool

    ' Set the current playback position of a channel.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL)
    ' pos    : the position
    '          if HCHANNEL: position in bytes
    '          if HMUSIC: LOWORD=order HIWORD=row ... use MAKELONG(order,row)
    '          if HSTREAM: position in bytes, file streams only
    '          if CDCHANNEL: position in milliseconds from start of track
    Declare Function BASS_ChannelSetPosition Lib "bass.dll" (ByVal handle As Integer, ByVal pos As Long) As BassBool

    ' Get the current playback position of a channel.
    ' handle : Channel handle (HCHANNEL/HMUSIC/HSTREAM, orelse CDCHANNEL)
    ' RETURN : the position (-1=error)
    '          if HCHANNEL: position in bytes
    '          if HMUSIC: LOWORD=order HIWORD=row (use GetLoWord(position), GetHiWord(Position))
    '          if HSTREAM: total bytes played since the stream was last flushed
    '          if CDCHANNEL: position in milliseconds from start of track
    Declare Function BASS_ChannelGetPosition Lib "bass.dll" (ByVal handle As Integer) As Long

    ' Calculate a channel's current output level.
    ' handle : channel handle(HMUSIC / HSTREAM, orelse RECORDCHAN)
    ' RETURN : LOWORD=left level (0-128) HIWORD=right level (0-128) (-1=error)
    '          Use GetLoWord and GetHiWord functions on return function.
    Declare Function BASS_ChannelGetLevel Lib "bass.dll" (ByVal handle As Integer) As Integer

    ' Retrieves upto "length" bytes of the channel's current sample data. This is
    ' useful if you wish to "visualize" the sound.
    ' handle:  Channel handle(HMUSIC / HSTREAM, orelse RECORDCHAN)
    ' buffer : Location to write the data
    ' length : Number of bytes wanted, orelse a BASS_DATA_xxx flag
    ' RETURN : Number of bytes actually written to the buffer (-1=error)
    Declare Function BASS_ChannelGetData Lib "bass.dll" (ByVal handle As Integer, ByRef buffer As Single, ByVal Length As Integer) As Integer
    Declare Function BASS_ChannelGetData Lib "bass.dll" (ByVal handle As Integer, ByRef buffer As Integer, ByVal Length As Integer) As Integer
    Declare Function BASS_ChannelGetData Lib "bass.dll" (ByVal handle As Integer, ByRef buffer As Short, ByVal Length As Integer) As Integer
    Declare Function BASS_ChannelGetData Lib "bass.dll" (ByVal handle As Integer, ByRef buffer As Byte, ByVal Length As Integer) As Integer

    ' Setup a sync on a channel. Multiple syncs may be used per channel.
    ' handle : Channel handle (currently there are only HMUSIC syncs)
    ' atype  : Sync type (BASS_SYNC_xxx type & flags)
    ' param  : Sync parameters (see the BASS_SYNC_xxx type description)
    ' proc   : User defined callback function (use AddressOf SYNCPROC)
    ' user   : The 'user' value passed to the callback function
    ' Return : Sync handle(Null = Error)
    Declare Function BASS_ChannelSetSync Lib "bass.dll" (ByVal handle As Integer, ByVal atype As SetSyncSyncType, ByVal param As Long, ByVal proc As SYNCPROC_Handler, ByVal user As Integer) As Integer

    ' Remove a sync from a channel
    ' handle : channel handle(HMUSIC)
    ' sync   : Handle of sync to remove
    Declare Function BASS_ChannelRemoveSync Lib "bass.dll" (ByVal handle As Integer, ByVal sync As Integer) As BassBool

    ' Setup a user DSP function on a channel. When multiple DSP functions
    ' are used on a channel, they are called in the order that they were added.
    ' handle:  channel handle(HMUSIC / HSTREAM)
    ' proc   : User defined callback function
    ' user   : The 'user' value passed to the callback function
    ' RETURN : DSP handle (NULL=error)
    Declare Function BASS_ChannelSetDSP Lib "bass.dll" (ByVal handle As Integer, ByVal proc As Integer, ByVal user As Integer) As Integer

    ' Remove a DSP function from a channel
    ' handle : channel handle(HMUSIC / HSTREAM)
    ' dsp    : Handle of DSP to remove */
    ' RETURN : BASSTRUE / BASSFALSE
    Declare Function BASS_ChannelRemoveDSP Lib "bass.dll" (ByVal handle As Integer, ByVal dsp As Integer) As BassBool

    ' Set the wet(reverb)/dry(no reverb) mix ratio on the channel. By default
    ' the distance of the sound from the listener is used to calculate the mix.
    ' NOTE: The channel must have 3D functionality enabled for the EAX environment
    ' to have any affect on it.
    ' handle : channel handle(HCHANNEL / HSTREAM / HMUSIC)
    ' mix    : The ratio (0.0=reverb off, 1.0=max reverb, -1.0=let EAX calculate
    '          the reverb mix based on the distance)
    Declare Function BASS_ChannelSetEAXMix Lib "bass.dll" (ByVal handle As Integer, ByVal mix As Single) As BassBool

    ' Get the wet(reverb)/dry(no reverb) mix ratio on the channel.
    ' handle:    channel handle(HCHANNEL / HSTREAM / HMUSIC)
    ' mix    : Pointer to store the ratio at
    Declare Function BASS_ChannelGetEAXMix Lib "bass.dll" (ByVal handle As Integer, ByRef mix As Single) As BassBool

    ' Set a link between 2 channels. When the 1st is played/stopped/paused/resumed
    ' the 2nd channel is also played/stopped/paused/resumed.
    ' handle : Handle of channel to link "chan" to (HMUSIC/HSTREAM)
    ' chan   : Handle of channel to link to "handle" (HMUSIC/HSTREAM)
    Declare Function BASS_ChannelSetLink Lib "bass.dll" (ByVal handle As Integer, ByVal chan As Integer) As BassBool

    ' Remove a link from a channel.
    ' handle : Handle of channel to unlink with "chan" (HMUSIC/HSTREAM)
    ' chan   : Handle of channel to unlink from "handle" (HMUSIC/HSTREAM)
    Declare Function BASS_ChannelRemoveLink Lib "bass.dll" (ByVal handle As Integer, ByVal chan As Integer) As BassBool

    ' Setup a DX8 effect on a channel. Can only be used when the channel
    ' is not playing. Use BASS_FXSetParameters to set the effect parameters.
    ' Obviously requires DX8.
    ' handle : Channel handle (HMUSIC/HSTREAM)
    ' atype   : Type of effect to setup (BASS_FX_xxx)
    ' RETURN : FX handle (NULL=error)
    Declare Function BASS_ChannelSetFX Lib "bass.dll" (ByVal handle As Integer, ByVal atype As Integer) As Integer

    ' Remove a DX8 effect from a channel. Can only be used when the
    ' channel is not playing.
    ' handle : Channel handle (HMUSIC/HSTREAM)
    ' fx     : Handle of FX to remove
    Declare Function BASS_ChannelRemoveFX Lib "bass.dll" (ByVal handle As Integer, ByVal fx As Integer) As BassBool

    ' Set the parameters of a DX8 effect.
    ' handle : FX handle
    ' par    : Pointer to the parameter structure
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As Integer) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXCHORUS) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXCOMPRESSOR) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXDISTORTION) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXECHO) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXFLANGER) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXGARGLE) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXI3DL2REVERB) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXPARAMEQ) As BassBool
    Declare Function BASS_FXSetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXREVERB) As BassBool

    ' Retrieve the parameters of a DX8 effect.
    ' handle : FX handle
    ' par    : Pointer to the parameter structure
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As Integer) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXCHORUS) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXCOMPRESSOR) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXDISTORTION) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXECHO) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXFLANGER) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXGARGLE) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXI3DL2REVERB) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXPARAMEQ) As BassBool
    Declare Function BASS_FXGetParameters Lib "bass.dll" (ByVal handle As Integer, ByRef par As BASS_FXREVERB) As BassBool

#End Region

#End Region

#Region " BASS Callback Delegate Functions "

    Public Delegate Function STREAMFILEPROC_Handler(ByVal action As Integer, ByVal param1 As Integer, ByVal param2 As Integer, ByVal user As Integer) As Integer

    Public Delegate Function STREAMPROC_Handler(ByVal handle As Integer, ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer) As Integer

    Public Delegate Sub SYNCPROC_Handler(ByVal handle As Integer, ByVal channel As Integer, ByVal data As Integer, ByVal user As Integer)

    Public Delegate Sub DSPPROC_Handler(ByVal handle As Integer, ByVal channel As Integer, ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer)

    Public Delegate Function RECORDPROC_Handler(ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer) As Integer

    Public Delegate Sub DOWNLOADPROC_Handler(ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer)

#End Region

#Region " BASS Callback Procedures "

    Function STREAMPROC(ByVal handle As Integer, ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer) As Integer

        'CALLBACK FUNCTION !!!

        'In here you can write a function to write out to a file, orelse send over the
        'internet etc, and stream into a BASS Buffer on the client, its up to you.
        'This function must return the number of bytes written out, so that BASS,
        'knows where to carry on sending from.

        ' NOTE: A stream function should obviously be as quick
        ' as possible, other streams (and MOD musics) can't be mixed until it's finished.
        ' handle : The stream that needs writing
        ' buffer : Buffer to write the samples in
        ' length : Number of bytes to write
        ' user   : The 'user' parameter value given when calling BASS_StreamCreate
        ' RETURN : Number of bytes written. If less than "length" then the
        '          stream is assumed to be at the end, and is stopped.

    End Function

    Function STREAMFILEPROC(ByVal action As Integer, ByVal param1 As Integer, ByVal param2 As Integer, ByVal user As Integer) As Integer

        'CALLBACK FUNCTION !!!

        ' User file stream callback function.
        ' action : The action to perform, one of BASS_FILE_xxx values.
        ' param1 : Depends on "action"
        ' param2 : Depends on "action"
        ' user   : The 'user' parameter value given when calling BASS_StreamCreate
        ' RETURN : Depends on "action"

    End Function

    Sub SYNCPROC(ByVal handle As Integer, ByVal channel As Integer, ByVal data As Integer, ByVal user As Integer)

        'CALLBACK FUNCTION !!!

        'Similarly in here, write what to do when sync function
        'is called, i.e screen flash etc.

        ' NOTE: a sync callback function should be very
        ' quick (eg. just posting a message) as other syncs cannot be processed
        ' until it has finished.
        ' handle : The sync that has occured
        ' channel: Channel that the sync occured in
        ' data   : Additional data associated with the sync's occurance
        ' user   : The 'user' parameter given when calling BASS_ChannelSetSync */

    End Sub

    Sub DSPPROC(ByVal handle As Integer, ByVal channel As Integer, ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer)

        'CALLBACK FUNCTION !!!

        ' VB doesn't support pointers, so you should copy the buffer into an array,
        ' process it, and then copy it back into the buffer.

        ' DSP callback function. NOTE: A DSP function should obviously be as quick as
        ' possible... other DSP functions, streams and MOD musics can not be processed
        ' until it's finished.
        ' handle : The DSP handle
        ' channel: Channel that the DSP is being applied to
        ' buffer : Buffer to apply the DSP to
        ' length : Number of bytes in the buffer
        ' user   : The 'user' parameter given when calling BASS_ChannelSetDSP

    End Sub

    Function RECORDPROC(ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer) As Integer

        'CALLBACK FUNCTION !!!

        ' Recording callback function.
        ' buffer : Buffer containing the recorded samples
        ' length : Number of bytes
        ' user   : The 'user' parameter value given when calling BASS_RecordStart
        ' RETURN : BASSTRUE = continue recording, BASSFALSE = stop

    End Function

    Sub DOWNLOADPROC(ByVal buffer As Integer, ByVal length As Integer, ByVal user As Integer)

        'CALLBACK FUNCTION !!!

        ' Download callback function.
        ' buffer : Buffer containing the recorded samples
        ' length : Number of bytes
        ' user   : The 'user' parameter value given when calling BASS_RecordStart
        ' RETURN : BASSTRUE = continue recording, BASSFALSE = stop

    End Sub

#End Region

#Region " Helper Functions "

    ' Gets the text description of a device. This function can be used to enumerate the available devices.
    ' devnum: The device(0 = First)
    ' Return: device description
    ' NOTE: use BASS_GetDeviceCount to get the number of available devices
    Public Function BASS_GetDeviceDescriptionString(ByVal devnum As Integer) As String
        Dim pstring As IntPtr
        Dim sstring As String
        On Error Resume Next
        pstring = BASS_GetDeviceDescription(devnum)
        If CBool(pstring.ToInt32) Then
            sstring = VBStrFromAnsiPtr(pstring)
        End If
        BASS_GetDeviceDescriptionString = sstring
    End Function

    ' Gets the number of available devices.
    ' Since the first device is 0, the last device is (BASS_GetDeviceCount - 1)
    Public Function BASS_GetDeviceCount() As Integer
        Dim count As Integer
        On Error Resume Next
        Do While CBool(BASS_GetDeviceDescription(count).ToInt32)
            count = count + 1
        Loop
        BASS_GetDeviceCount = count
    End Function

    'Retrieves a music's name.
    'handle :  Music handle
    'Return:   Music's name
    Public Function BASS_MusicGetNameString(ByVal handle As Integer) As String
        Dim pstring As IntPtr
        Dim sstring As String
        On Error Resume Next
        pstring = BASS_MusicGetName(handle)
        If CBool(pstring.ToInt32) Then
            sstring = VBStrFromAnsiPtr(pstring)
        End If
        BASS_MusicGetNameString = sstring
    End Function

    Public Function BASS_SetEAXParametersVB(ByRef Preset As Object) As Integer
        ' This function is a workaround, because VB doesn't support multiple comma seperated
        ' paramaters for each Global Const, simply pass the EAX_PRESET_XXXX value to this function
        ' instead of BASS_SetEasParamaets as you would to in C++
        Select Case Preset
            Case EAX_PRESET_GENERIC
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_GENERIC, 0.5, 1.493, 0.5)
            Case EAX_PRESET_PADDEDCELL
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_PADDEDCELL, 0.25, 0.1, 0)
            Case EAX_PRESET_ROOM
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_ROOM, 0.417, 0.4, 0.666)
            Case EAX_PRESET_BATHROOM
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_BATHROOM, 0.653, 1.499, 0.166)
            Case EAX_PRESET_LIVINGROOM
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_LIVINGROOM, 0.208, 0.478, 0)
            Case EAX_PRESET_STONEROOM
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_STONEROOM, 0.5, 2.309, 0.888)
            Case EAX_PRESET_AUDITORIUM
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_AUDITORIUM, 0.403, 4.279, 0.5)
            Case EAX_PRESET_CONCERTHALL
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_CONCERTHALL, 0.5, 3.961, 0.5)
            Case EAX_PRESET_CAVE
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_CAVE, 0.5, 2.886, 1.304)
            Case EAX_PRESET_ARENA
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_ARENA, 0.361, 7.284, 0.332)
            Case EAX_PRESET_HANGAR
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_HANGAR, 0.5, 10, 0.3)
            Case EAX_PRESET_CARPETEDHALLWAY
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_CARPETEDHALLWAY, 0.153, 0.259, 2)
            Case EAX_PRESET_HALLWAY
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_HALLWAY, 0.361, 1.493, 0)
            Case EAX_PRESET_STONECORRIDOR
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_STONECORRIDOR, 0.444, 2.697, 0.638)
            Case EAX_PRESET_ALLEY
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_ALLEY, 0.25, 1.752, 0.776)
            Case EAX_PRESET_FOREST
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_FOREST, 0.111, 3.145, 0.472)
            Case EAX_PRESET_CITY
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_CITY, 0.111, 2.767, 0.224)
            Case EAX_PRESET_MOUNTAINS
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_MOUNTAINS, 0.194, 7.841, 0.472)
            Case EAX_PRESET_QUARRY
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_QUARRY, 1, 1.499, 0.5)
            Case EAX_PRESET_PLAIN
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_PLAIN, 0.097, 2.767, 0.224)
            Case EAX_PRESET_PARKINGLOT
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_PARKINGLOT, 0.208, 1.652, 1.5)
            Case EAX_PRESET_SEWERPIPE
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_SEWERPIPE, 0.652, 2.886, 0.25)
            Case EAX_PRESET_UNDERWATER
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_UNDERWATER, 1, 1.499, 0)
            Case EAX_PRESET_DRUGGED
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_DRUGGED, 0.875, 8.392, 1.388)
            Case EAX_PRESET_DIZZY
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_DIZZY, 0.139, 17.234, 0.666)
            Case EAX_PRESET_PSYCHOTIC
                BASS_SetEAXParametersVB = BASS_SetEAXParameters(EAX_ENVIRONMENT_PSYCHOTIC, 0.486, 7.563, 0.806)
        End Select
    End Function

    Function BASS_GetStringVersion() As String
        'This function will return the string version
        'of the BASS DLL. For example the provided function within the DLL
        '"BASS_GetVersion" will return 393216, whereas this function works
        'out the actual version string as you would need to see it.
        BASS_GetStringVersion = Trim(Str(GetLoWord(BASS_GetVersion))) & "." & Trim(Str(GetHiWord(BASS_GetVersion)))
    End Function

    Public Function GetHiWord(ByRef lparam As Integer) As Integer
        ' This is the HIWORD of the lParam:
        GetHiWord = lparam \ &H10000 And &HFFFF
    End Function
    Public Function GetLoWord(ByRef lparam As Integer) As Integer
        ' This is the LOWORD of the lParam:
        GetLoWord = lparam And &HFFFF
    End Function

    Public Function BASS_GetErrorDescription(ByRef ErrorCode As Bass_ErrorCodes) As String
        Select Case ErrorCode
            Case Bass_ErrorCodes.Bass_Error_Ok
                Return "All is OK"
            Case Bass_ErrorCodes.Bass_Error_Mem
                Return "Memory Error"
            Case Bass_ErrorCodes.Bass_Error_Fileopen
                Return "Can't Open the File"
            Case Bass_ErrorCodes.Bass_Error_Driver
                Return "Can't Find a Free Sound Driver"
            Case Bass_ErrorCodes.Bass_Error_Buflost
                Return "The Sample Buffer Was Lost - Please Report This!"
            Case Bass_ErrorCodes.Bass_Error_Handle
                Return "Invalid Handle"
            Case Bass_ErrorCodes.Bass_Error_Format
                Return "Unsupported Format"
            Case Bass_ErrorCodes.Bass_Error_Position
                Return "Invalid Playback Position"
            Case Bass_ErrorCodes.Bass_Error_Init
                Return "BASS_Init Has Not Been Successfully Called"
            Case Bass_ErrorCodes.Bass_Error_Start
                Return "BASS_Start Has Not Been Successfully Called"
            Case Bass_ErrorCodes.Bass_Error_Already
                Return "Already Initialized"
            Case Bass_ErrorCodes.Bass_Error_Nopause
                Return "Not Paused"
            Case Bass_ErrorCodes.Bass_Error_Notaudio
                Return "Not An Audio Track"
            Case Bass_ErrorCodes.Bass_Error_Nochan
                Return "Can't Get a Free Channel"
            Case Bass_ErrorCodes.Bass_Error_Illtype
                Return "An Illegal Type Was Specified"
            Case Bass_ErrorCodes.Bass_Error_Illparam
                Return "An Illegal Parameter Was Specified"
            Case Bass_ErrorCodes.Bass_Error_No3D
                Return "No 3D Support"
            Case Bass_ErrorCodes.Bass_Error_Noeax
                Return "No EAX Support"
            Case Bass_ErrorCodes.Bass_Error_Device
                Return "Illegal Device Number"
            Case Bass_ErrorCodes.Bass_Error_Noplay
                Return "Not Playing"
            Case Bass_ErrorCodes.Bass_Error_Freq
                Return "Illegal Sample Rate"
            Case Bass_ErrorCodes.Bass_Error_Notfile
                Return "The Stream is Not a File Stream (WAV/MP3)"
            Case Bass_ErrorCodes.Bass_Error_Nohw
                Return "No Hardware Voices Available"
            Case Bass_ErrorCodes.Bass_Error_Empty
                Return "The MOD music has no sequence data"
            Case Bass_ErrorCodes.Bass_Error_Nonet
                Return "No Internet connection could be opened"
            Case Bass_ErrorCodes.Bass_Error_Create
                Return "Couldn't create the file"
            Case Bass_ErrorCodes.Bass_Error_Nofx
                Return "Effects are not enabled"
            Case Bass_ErrorCodes.Bass_Error_Playing
                Return "The channel is playing"
            Case Bass_ErrorCodes.Bass_Error_Notavail
                Return "The requested data is not available"
            Case Bass_ErrorCodes.Bass_Error_Decode
                Return "The channel is a 'decoding channel'"
            Case Bass_ErrorCodes.Bass_Error_Unknown
                Return "Some Other Mystery Error"
        End Select
    End Function

    Public Function MakeLong(ByRef LoWord As Integer, ByRef HiWord As Integer) As Integer
        'Replacement for the c++ Function MAKELONG
        'You need this to pass values to certain function calls.
        'i.e BASS_ChannelSetPosition needs to pass a value
        'using make long, i.e BASS_ChannelSetPosition Handle,MakeLong(Order,Row)
        MakeLong = LoWord Or LShift(HiWord, 16)
    End Function

    Public Function LShift(ByVal lValue As Integer, ByVal lNumberOfBitsToShift As Integer) As Integer
        Const ksCallname As String = "LShift"
        On Error GoTo Procedure_Error
        LShift = CInt(lValue * (2 ^ lNumberOfBitsToShift))

Procedure_Exit:
        Exit Function

Procedure_Error:
        Err.Raise(Err.Number, ksCallname, Err.Description, Err.HelpFile, Err.HelpContext)
        Resume Procedure_Exit
    End Function

    Public Function RShift(ByVal lValue As Integer, ByVal lNumberOfBitsToShift As Integer) As Integer

        Const ksCallname As String = "RShift"
        On Error GoTo Procedure_Error
        RShift = CInt(lValue \ CLng((2 ^ lNumberOfBitsToShift)))

Procedure_Exit:
        Exit Function

Procedure_Error:
        Err.Raise(Err.Number, ksCallname, Err.Description, Err.HelpFile, Err.HelpContext)
        Resume Procedure_Exit
    End Function

    Public Function VBStrFromAnsiInteger(ByVal lpStr As Integer) As String
        ' Wow, rewrote with one line.

        Return Marshal.PtrToStringAnsi(New IntPtr(lpStr))
    End Function

    Public Function VBStrFromAnsiPtr(ByVal lpStr As IntPtr) As String
        ' Wow, rewrote with one line.

        Return Marshal.PtrToStringAnsi(lpStr)
    End Function

    'Formats a string with mm:ss for channel.
    'inPos :  Position / Length of channel
    'Retuns: String in format MM:SS
    Public Function FixTimespan(ByVal inPos As Double) As String
        Return Mid(TimeSpan.FromSeconds(inPos).ToString, 4, 5)
    End Function

    Public Function GetChannelDescription(ByVal chan As Integer) As String
        Select Case chan
            Case 1
                Return chan.ToString & " (Mono)"
            Case 2
                Return chan.ToString & " (Stereo)"
            Case Else
                Return chan.ToString
        End Select
    End Function
#End Region

    Public Declare Function TAGS_Read Lib "tags.dll" Alias "TAGS_Read" (ByVal handle As Integer, ByVal fmt As String) As String
    Public Declare Function TAGS_GetLastErrorDesc Lib "tags.dll" Alias "TAGS_GetLastErrorDesc" () As Integer
    Public Declare Function TAGS_GetVersion Lib "tags.dll" Alias "TAGS_GetVersion" () As Integer


End Class


